from typing import TYPE_CHECKING

from . import viewer
from .coughdrop_processor import CoughDropProcessor
from .dot_processor import DotProcessor
from .gridset_processor import GridsetProcessor
from .opml_processor import OPMLProcessor
from .snap_processor import SnapProcessor
from .touchchat_processor import TouchChatProcessor
from .tree_structure import AACButton, AACPage, AACTree, ButtonType

if TYPE_CHECKING:
    from .optional.screenshot_processor import ScreenshotProcessor

__all__ = [
    "GridsetProcessor",
    "TouchChatProcessor",
    "SnapProcessor",
    "CoughDropProcessor",
    "OPMLProcessor",
    "DotProcessor",
    "AACTree",
    "AACPage",
    "AACButton",
    "ButtonType",
    "viewer",
    "ScreenshotProcessor",
    "get_screenshot_processor",
]


def get_screenshot_processor() -> "ScreenshotProcessor":
    """Lazy load the screenshot processor.

    Returns:
        Initialized ScreenshotProcessor instance

    Raises:
        ImportError: If screenshot dependencies are not installed
    """
    try:
        from .optional.screenshot_processor import ScreenshotProcessor

        return ScreenshotProcessor()
    except ImportError as e:
        raise ImportError(
            "Screenshot processing requires additional dependencies. "
            "Install them with: pip install aac-processors[screenshot]"
        ) from e



import re
from typing import Any, Optional

from aac_processors.base_processor import AACProcessor
from aac_processors.tree_structure import AACButton, AACPage, AACTree, ButtonType


class DotProcessor(AACProcessor):
    """Processor for DOT (Graph Description Language) files."""

    def __init__(self) -> None:
        """Initialize the processor."""
        super().__init__()
        self.debug = False
        self.file_path = ""
        self.original_file_path = ""

    def can_process(self, file_path: str) -> bool:
        """Check if file is a DOT file.

        Args:
            file_path: Path to the file to check.

        Returns:
            True if file is a DOT file.
        """
        return file_path.lower().endswith((".dot", ".gv"))

    def set_source_file(self, file_path: str) -> None:
        """Set the source file path.

        Args:
            file_path: Path to the source file.
        """
        self.file_path = file_path
        if not self.original_file_path:
            self.original_file_path = file_path

    def load_into_tree(self, file_path: str) -> AACTree:
        """Load a DOT file into an AAC tree structure.

        Args:
            file_path: Path to the DOT file.

        Returns:
            AACTree structure.
        """
        self.set_source_file(file_path)

        # Read DOT file
        with open(file_path) as f:
            dot_content = f.read()

        print(f"DOT content: {dot_content}")

        # Parse DOT content
        nodes, edges = self._parse_dot(dot_content)

        print(f"Parsed nodes: {nodes}")
        print(f"Parsed edges: {edges}")

        # Create tree structure
        tree = AACTree()

        # Add pages
        for node_id, label in nodes.items():
            # Create a page for each node
            page = AACPage(
                id=node_id,
                name=label,  # Use the label as the page name
                buttons=[],
                grid_size=(3, 3),  # Default grid size
            )
            tree.add_page(page)

        # Add buttons (edges)
        for source, target, edge_label in edges:
            if source in tree.pages and target in tree.pages:
                # Create a button for each edge
                button_label = (
                    edge_label or f"Go to {nodes[target]}"
                )  # Use the actual node label
                button = AACButton(
                    id=f"button_{source}_{target}",
                    label=button_label,
                    type=ButtonType.NAVIGATE,
                    target_page_id=target,
                )
                tree.pages[source].buttons.append(button)

        print(f"Tree pages: {[p.name for p in tree.pages.values()]}")

        return tree

    def save_from_tree(self, tree: AACTree, file_path: str) -> None:
        """Save an AAC tree to a DOT file.

        Args:
            tree: AACTree structure.
            file_path: Path to save the DOT file.
        """
        # Create DOT content
        dot_lines = ["digraph G {"]

        # Add nodes
        node_counter = 1
        node_id_map = {}  # Map page_id to node_id

        for page_id, page in tree.pages.items():
            # Use consistent node IDs
            node_id = f"node{node_counter}"
            node_id_map[page_id] = node_id
            node_counter += 1

            # Create a node for each page
            dot_lines.append(f'    {node_id} [label="{page.name}"];')

        # Add edges
        for page_id, page in tree.pages.items():
            source_id = node_id_map[page_id]

            for button in page.buttons:
                if button.type == ButtonType.NAVIGATE and button.target_page_id:
                    if button.target_page_id in node_id_map:
                        target_id = node_id_map[button.target_page_id]
                        # Create an edge for each navigation button
                        dot_lines.append(
                            f'    {source_id} -> {target_id} [label="{button.label}"];'
                        )
                elif button.type == ButtonType.SPEAK:
                    # For speak buttons, create a node for the button itself
                    speak_node_id = f"node{node_counter}"
                    node_counter += 1
                    dot_lines.append(f'    {speak_node_id} [label="{button.label}"];')
                    dot_lines.append(
                        f'    {source_id} -> {speak_node_id} [label="{button.label}"];'
                    )

        dot_lines.append("}")

        # Write to file
        with open(file_path, "w") as f:
            f.write("\n".join(dot_lines))

    def export_tree(self, tree: AACTree, output_path: str) -> None:
        """Export tree to DOT format.

        Args:
            tree (AACTree): Tree to export.
            output_path (str): Path where to save the file.
        """
        self.save_from_tree(tree, output_path)

    def extract_texts(self, file_path: str) -> list[str]:
        """Extract translatable texts from DOT file.

        Args:
            file_path: Path to the DOT file.

        Returns:
            List of translatable texts.
        """
        texts = []

        try:
            # Read DOT file
            with open(file_path) as f:
                dot_content = f.read()

            # Extract node labels
            node_pattern = r'(\w+)\s*\[.*?label\s*=\s*["\']([^"\']*)["\'].*?\]'
            for match in re.finditer(node_pattern, dot_content):
                label = match.group(2)
                if label and label.strip():
                    texts.append(label.strip())

            # Extract edge labels
            edge_pattern = (
                r'(\w+)\s*(-[->])\s*(\w+)\s*\[.*?label\s*=\s*["\']([^"\']*)["\'].*?\]'
            )
            for match in re.finditer(edge_pattern, dot_content):
                label = match.group(4)
                if label and label.strip():
                    texts.append(label.strip())

        except Exception as e:
            self._debug_print(f"Error extracting texts from DOT file: {str(e)}")

        return list(set(texts))  # Remove duplicates

    def process_texts(
        self,
        file_path: str,
        translations: Optional[dict[str, Any]] = None,
        output_path: Optional[str] = None,
    ) -> Optional[str]:
        """Process texts in a DOT file.

        Args:
            file_path: Path to the DOT file.
            translations: Optional translations to apply.
            output_path: Optional path to save the translated file.

        Returns:
            Optional path to the translated file.
        """
        try:
            # Check if we can process this file
            if not self.can_process(file_path):
                return None

            # If no translations, just return None
            if not translations:
                return None

            # Extract texts from the DOT file
            texts = self.extract_texts(file_path)

            # Apply translations
            translated_texts = {}
            for text in texts:
                if text in translations:
                    translated_texts[text] = translations[text]
                else:
                    translated_texts[text] = text  # Default to original text

            # Create translated file if output path is provided
            if output_path and translated_texts:
                self.create_translated_file(file_path, translated_texts, output_path)
                return output_path

            return None
        except Exception as e:
            self._debug_print(f"Error translating DOT file: {str(e)}")
            return None

    def process_files(
        self, directory: str, translations: Optional[dict[str, Any]] = None
    ) -> Optional[str]:
        """Process files in directory.

        Args:
            directory: Directory containing files to process.
            translations: Dictionary of translations.

        Returns:
            Path to translated file if successful, None otherwise.
        """
        import os

        for root, _, files in os.walk(directory):
            for file in files:
                abs_path = os.path.join(root, file)
                if self.can_process(abs_path):
                    output_path = None
                    if translations:
                        # Generate output path in the same directory
                        target_lang = translations.get("target_lang")
                        filename = os.path.basename(abs_path)
                        if target_lang:
                            base, ext = os.path.splitext(filename)
                            translated_filename = f"{base}_{target_lang}{ext}"
                        else:
                            translated_filename = f"translated_{filename}"
                        output_path = os.path.join(directory, translated_filename)

                    result = self.process_texts(abs_path, translations, output_path)
                    if result and translations:
                        return result

        return None

    def get_dot_text(self, file_path: str) -> str:
        """Get the DOT text from a file.

        Args:
            file_path: Path to the DOT file.

        Returns:
            DOT text.
        """
        with open(file_path) as f:
            return f.read()

    def _debug_print(self, message: str) -> None:
        """Print debug message if debug is enabled.

        Args:
            message: Debug message to print.
        """
        if self.debug:
            print(f"[DotProcessor] {message}")

    def create_translated_file(
        self, input_file: str, translations: dict, output_path: str
    ) -> Optional[str]:
        """Create a translated version of the DOT file.

        Args:
            input_file: Path to the input file.
            translations: Dictionary of translations.
            output_path: Path to save the translated file.

        Returns:
            Path to the translated file if successful, otherwise None.
        """
        try:
            # Load the DOT file
            with open(input_file) as f:
                content = f.read()

            # Apply translations to all text in the file
            translated_content = content
            for original, translated in translations.items():
                if original != "target_lang" and original != "output_path":
                    # Replace the text in labels, being careful with quotes
                    pattern = f'label="({re.escape(original)})"'
                    replacement = f'label="{translated}"'
                    translated_content = re.sub(
                        pattern, replacement, translated_content
                    )

            # Write the translated content to the output file
            with open(output_path, "w") as f:
                f.write(translated_content)

            return output_path
        except Exception as e:
            if self._debug_output:
                self._debug_output(f"Error creating translated file: {e}")
            return None

    def _parse_dot(
        self, dot_content: str
    ) -> tuple[dict[str, str], list[tuple[str, str, str]]]:
        """Parse DOT content to extract nodes and edges.

        Args:
            dot_content: Content of the DOT file.

        Returns:
            Tuple containing:
                - Dictionary mapping node IDs to labels
                - List of edge tuples (source, target, edge_label)
        """
        # First, clean up the DOT content by removing comments
        lines = []
        for line in dot_content.split("\n"):
            line = line.strip()
            if line and not line.startswith("//") and not line.startswith("#"):
                lines.append(line)

        # Process line by line
        nodes = {}
        edges = []

        for line in lines:
            # Check if it's a node definition with explicit label
            # Pattern: node1 [label="Home Page"];
            node_match = re.match(
                r'\s*(\w+|\".+?\")\s*\[\s*label\s*=\s*["\']([^"\']*)["\'].*?\];?', line
            )
            if node_match:
                node_id = node_match.group(1).strip('"')
                label = node_match.group(2)
                nodes[node_id] = label
                continue

            # Check if it's an edge definition
            # Pattern: "more quick chat" -> "That tickles";
            # Or: node1 -> node2 [label="Go to About"];
            edge_match = re.match(
                r"\s*(\w+|\".+?\")\s*->\s*(\w+|\".+?\")"
                r'(?:\s*\[\s*label\s*=\s*["\']([^"\']*)["\'].*?\])?;?',
                line,
            )
            if edge_match:
                source = edge_match.group(1).strip('"')
                target = edge_match.group(2).strip('"')
                edge_label = edge_match.group(3) if edge_match.group(3) else ""

                # Add nodes if they don't exist yet (implicit node definition)
                if source not in nodes:
                    nodes[source] = source
                if target not in nodes:
                    nodes[target] = target

                edges.append((source, target, edge_label))

        # Debug output
        print(f"Parsed nodes: {nodes}")
        print(f"Parsed edges: {edges}")

        return nodes, edges



import json
import os
import tempfile
import zipfile

import pytest

from aac_processors.coughdrop_processor import CoughDropProcessor
from aac_processors.tree_structure import ButtonType


@pytest.fixture
def processor():
    return CoughDropProcessor()


@pytest.fixture
def sample_obf_data():
    return {
        "format": "open-board-0.1",
        "id": "test_board",
        "locale": "en",
        "name": "Test Board",
        "grid": {
            "rows": 2,
            "columns": 2,
            "order": [["btn1", "btn2"], ["btn3", "btn4"]],
        },
        "buttons": [
            {
                "id": "btn1",
                "label": "Hello",
                "vocalization": "Hello there!",
                "image_id": "img1",
            },
            {
                "id": "btn2",
                "label": "More",
                "load_board": {"id": "board2", "path": "boards/board2.obf"},
            },
            {"id": "btn3", "label": "Clear", "action": ":clear"},
            {"id": "btn4", "label": "A", "action": "+a"},
        ],
        "images": [
            {
                "id": "img1",
                "url": "http://example.com/hello.png",
                "width": 300,
                "height": 300,
                "content_type": "image/png",
            }
        ],
    }


@pytest.fixture
def sample_obf_file(sample_obf_data):
    with tempfile.NamedTemporaryFile(mode="w", suffix=".obf", delete=False) as f:
        json.dump(sample_obf_data, f)
    yield f.name
    os.unlink(f.name)


@pytest.fixture
def sample_obz_file(sample_obf_data):
    with tempfile.NamedTemporaryFile(suffix=".obz", delete=False) as f:
        with zipfile.ZipFile(f.name, "w", zipfile.ZIP_DEFLATED) as zf:
            # Add manifest
            manifest = {
                "format": "open-board-0.1",
                "root": "boards/test_board.obf",
                "paths": {
                    "boards": {
                        "test_board": "boards/test_board.obf",
                        "board2": "boards/board2.obf",
                    }
                },
            }
            zf.writestr("manifest.json", json.dumps(manifest))

            # Add main board
            zf.writestr("boards/test_board.obf", json.dumps(sample_obf_data))

            # Add secondary board
            board2_data = {
                "format": "open-board-0.1",
                "id": "board2",
                "name": "Second Board",
                "grid": {"rows": 1, "columns": 1},
                "buttons": [
                    {
                        "id": "btn1",
                        "label": "Back",
                        "load_board": {
                            "id": "test_board",
                            "path": "boards/test_board.obf",
                        },
                    }
                ],
            }
            zf.writestr("boards/board2.obf", json.dumps(board2_data))
    yield f.name
    os.unlink(f.name)


def test_can_process(processor):
    """Test file type detection"""
    assert processor.can_process("test.obf") is True
    assert processor.can_process("test.obz") is True
    assert processor.can_process("test.txt") is False


def test_load_single_board(processor, sample_obf_file):
    """Test loading a single OBF file"""
    tree = processor.load_into_tree(sample_obf_file)

    assert len(tree.pages) == 1
    page = tree.pages["test_board"]
    assert page.name == "Test Board"
    assert page.grid_size == (2, 2)
    assert len(page.buttons) == 4

    # Check button types and properties
    speak_btn = next(b for b in page.buttons if b.id == "btn1")
    assert speak_btn.type == ButtonType.SPEAK
    assert speak_btn.label == "Hello"
    assert speak_btn.vocalization == "Hello there!"

    nav_btn = next(b for b in page.buttons if b.id == "btn2")
    assert nav_btn.type == ButtonType.NAVIGATE
    assert nav_btn.label == "More"
    assert nav_btn.target_page_id == "board2"

    action_btn = next(b for b in page.buttons if b.id == "btn3")
    assert action_btn.type == ButtonType.ACTION
    assert action_btn.label == "Clear"
    assert action_btn.action == ":clear"

    spell_btn = next(b for b in page.buttons if b.id == "btn4")
    assert spell_btn.type == ButtonType.ACTION
    assert spell_btn.label == "A"
    assert spell_btn.action == "+a"


def test_load_board_set(processor, sample_obz_file):
    """Test loading an OBZ file with multiple boards"""
    tree = processor.load_into_tree(sample_obz_file)

    assert len(tree.pages) == 2
    assert "test_board" in tree.pages
    assert "board2" in tree.pages

    # Check main board
    main_page = tree.pages["test_board"]
    assert main_page.name == "Test Board"
    assert len(main_page.buttons) == 4

    # Check secondary board
    second_page = tree.pages["board2"]
    assert second_page.name == "Second Board"
    assert len(second_page.buttons) == 1
    back_btn = second_page.buttons[0]
    assert back_btn.type == ButtonType.NAVIGATE
    assert back_btn.target_page_id == "test_board"


def test_extract_texts(processor, sample_obf_file):
    """Test text extraction from OBF file"""
    texts = processor.extract_texts(sample_obf_file)
    expected = ["Test Board", "Hello", "Hello there!", "More", "Clear", "A"]
    assert sorted(texts) == sorted(expected)


def test_create_translated_file(processor, sample_obf_file):
    """Test translation of OBF file"""
    translations = {
        "Test Board": "Tablero de Prueba",
        "Hello": "Hola",
        "Hello there!": "¡Hola!",
        "More": "Más",
        "Clear": "Borrar",
        "A": "A",
        "target_lang": "es",
    }

    result = processor.create_translated_file(sample_obf_file, translations)
    assert result is not None

    # Load and verify translated file
    tree = processor.load_into_tree(result)
    page = tree.pages["test_board"]
    assert page.name == "Tablero de Prueba"

    hello_btn = next(b for b in page.buttons if b.id == "btn1")
    assert hello_btn.label == "Hola"
    assert hello_btn.vocalization == "¡Hola!"


def test_process_texts_obz(processor, sample_obz_file):
    """Test processing texts in OBZ file"""
    # Test extraction
    texts = processor.process_texts(sample_obz_file)
    assert isinstance(texts, list)
    assert "Test Board" in texts
    assert "Second Board" in texts
    assert "Back" in texts

    # Test translation
    translations = {
        "Test Board": "Tablero de Prueba",
        "Second Board": "Segundo Tablero",
        "Back": "Volver",
        "target_lang": "es",
    }

    with tempfile.NamedTemporaryFile(suffix=".obz") as output:
        result = processor.process_texts(sample_obz_file, translations, output.name)
        assert result == output.name

        # Verify translated file
        with zipfile.ZipFile(output.name, "r") as zf:
            assert "manifest.json" in zf.namelist()
            assert "boards/test_board.obf" in zf.namelist()
            assert "boards/board2.obf" in zf.namelist()

            # Check main board translation
            main_board = json.loads(zf.read("boards/test_board.obf"))
            assert main_board["name"] == "Tablero de Prueba"

            # Check second board translation
            board2 = json.loads(zf.read("boards/board2.obf"))
            assert board2["name"] == "Segundo Tablero"
            assert board2["buttons"][0]["label"] == "Volver"



"""Processor for Apple Panels format (.ascconfig)."""

import os
import plistlib
from pathlib import Path
from typing import Any, Optional

from .file_processor import FileProcessor
from .tree_structure import AACButton, AACPage, AACTree, ButtonStyle, ButtonType


class ApplePanelsProcessor(FileProcessor):
    """Process Apple Panels format (.ascconfig folders)."""

    def __init__(self) -> None:
        """Initialize the processor."""
        super().__init__()
        self.collected_texts = []
        self.file_path: Optional[str] = None
        self.original_filename: Optional[str] = None
        self.original_file_path: Optional[str] = None

    def can_process(self, file_path: str) -> bool:
        """Check if file is an Apple Panels config.

        Args:
            file_path: Path to check

        Returns:
            True if path ends with .ascconfig
        """
        return file_path.lower().endswith(".ascconfig")

    def _load_plist(self, file_path: str) -> dict[str, Any]:
        """Load and parse a plist file.

        Args:
            file_path: Path to plist file

        Returns:
            Parsed plist data as dictionary
        """
        with open(file_path, "rb") as f:
            return plistlib.load(f)

    def _extract_button_info(
        self, button_dict: dict[str, Any]
    ) -> tuple[str, str, str, tuple[int, int]]:
        """Extract button information from button dictionary.

        Args:
            button_dict: Dictionary containing button data

        Returns:
            Tuple of (id, text, color, position)
        """
        # Extract button ID
        button_id = button_dict.get("ID", "")

        # Get display text
        text = button_dict.get("DisplayText", "")

        # Get color (default to white if not specified)
        color = button_dict.get("DisplayColor", "1.000 1.000 1.000 1.000")

        # Parse rect string like "{{x, y}, {w, h}}"
        rect_str = button_dict.get("Rect", "{{0, 0}, {0, 0}}")
        try:
            # Remove braces and split
            rect_parts = rect_str.replace("{", "").replace("}", "").split(",")
            x = int(float(rect_parts[0]))
            y = int(float(rect_parts[1]))
            # Calculate grid position based on coordinates
            # Assuming standard button size of 100x25
            row = y // 100
            col = x // 100
        except (ValueError, IndexError):
            row, col = 0, 0

        return button_id, text, color, (row, col)

    def _convert_color(self, color_str: str) -> str:
        """Convert color string from Apple format to hex.

        Args:
            color_str: Color string in format "r g b a" with values 0-1

        Returns:
            Hex color string like "#RRGGBB"
        """
        try:
            # Split color components and convert to 0-255 range
            r, g, b, _ = [float(x) for x in color_str.split()]
            return f"#{int(r * 255):02x}{int(g * 255):02x}{int(b * 255):02x}"
        except (ValueError, IndexError):
            return "#ffffff"  # Default to white

    def load_into_tree(self, file_path: str) -> AACTree:
        """Load Apple Panels config into tree structure.

        Args:
            file_path: Path to .ascconfig folder

        Returns:
            Tree structure containing pages and buttons
        """
        tree = AACTree()

        # Check paths
        contents_dir = Path(file_path) / "Contents"
        if not contents_dir.exists():
            msg = f"Invalid Apple Panels config - no Contents directory in {file_path}"
            raise ValueError(msg)

        # Load Info.plist
        info_path = contents_dir / "Info.plist"
        if not info_path.exists():
            raise ValueError(f"Missing Info.plist in {file_path}")

        self._load_plist(str(info_path))

        # Load panel definitions
        panels_path = contents_dir / "Resources" / "PanelDefinitions.plist"
        if not panels_path.exists():
            raise ValueError(f"Missing PanelDefinitions.plist in {file_path}")

        panels = self._load_plist(str(panels_path))

        # Process each panel
        for panel_id, panel_data in panels.get("Panels", {}).items():
            # Create page for panel
            page = AACPage(
                id=panel_id,
                name=panel_data.get("Name", "Untitled Panel"),
                grid_size=(0, 0),  # Will be calculated from buttons
            )

            # Process buttons
            max_row = 0
            max_col = 0
            for button_data in panel_data.get("PanelObjects", []):
                if button_data.get("PanelObjectType") == "Button":
                    # Extract button info
                    btn_id, text, color, pos = self._extract_button_info(button_data)
                    row, col = pos

                    # Track grid size
                    max_row = max(max_row, row)
                    max_col = max(max_col, col)

                    # Create button
                    btn = AACButton(
                        id=btn_id,
                        label=text,
                        type=ButtonType.SPEAK,  # Default to speak
                        position=pos,
                        style=ButtonStyle(body_color=self._convert_color(color)),
                    )

                    # Check for navigation action
                    actions = button_data.get("Actions", [])
                    for action in actions:
                        if action.get("ActionType") == "ActionOpenPanel":
                            btn.type = ButtonType.NAVIGATE
                            target = action.get("ActionParam", {})
                            btn.target_page_id = target.get("PanelID")
                        elif action.get("ActionType") == "ActionPressKeyCharSequence":
                            params = action.get("ActionParam", {})
                            btn.vocalization = params.get("CharString", text)

                    page.buttons.append(btn)

            # Update grid size
            page.grid_size = (max_row + 1, max_col + 1)

            # Add page to tree
            tree.add_page(page)

        return tree

    def save_from_tree(self, tree: AACTree, output_path: str) -> None:
        """Save tree structure as Apple Panels config.

        Args:
            tree: Tree structure to save
            output_path: Path to save to (must end with .ascconfig)
        """
        if not output_path.endswith(".ascconfig"):
            output_path += ".ascconfig"

        # Create directory structure
        config_dir = Path(output_path)
        contents_dir = config_dir / "Contents"
        resources_dir = contents_dir / "Resources"
        os.makedirs(resources_dir, exist_ok=True)

        # Create Info.plist
        stem = Path(output_path).stem
        info = {
            "ASCConfigurationDisplayName": "Converted Configuration",
            "ASCConfigurationIdentifier": str(stem),
            "ASCConfigurationProductSupportType": "VirtualKeyboard",
            "ASCConfigurationVersion": "7.1",
            "CFBundleDevelopmentRegion": "en",
            "CFBundleIdentifier": f"com.apple.AssistiveControl.panel.{stem}",
            "CFBundleName": "Assistive Control Panels",
            "CFBundleShortVersionString": "2.0",
            "CFBundleVersion": "1",
            "NSHumanReadableCopyright": "Generated by AAC Processors",
        }

        with open(contents_dir / "Info.plist", "wb") as f:
            plistlib.dump(info, f)

        # Create AssetIndex.plist for images
        assets = {}

        # Convert pages to panels
        panels = {
            "Panels": {},
            "ToolbarOrdering": {
                "ToolbarIdentifiersAfterBasePanel": [],
                "ToolbarIdentifiersPriorToBasePanel": [],
            },
        }

        # Convert pages to panels
        for page in tree.pages.values():
            panel = {
                "DisplayOrder": 1,
                "GlidingLensSize": 5,
                "HasTransientPosition": False,
                "HideHome": False,
                "HideMinimize": False,
                "HidePanelAdjustments": False,
                "HideSwitchDock": False,
                "HideSwitchDockContextualButtons": False,
                "HideTitlebar": False,
                "ID": page.id,
                "Name": page.name,
                "PanelObjects": [],
                "ProductSupportType": "All",
                "Rect": "{{15, 75}, {425, 55}}",
                "ScanStyle": 0,
                "ShowPanelLocationString": "CustomPanelList",
                "UsesPinnedResizing": False,
            }

            # Get page dimensions
            rows, cols = page.grid_size
            page_width = 1100  # Default width
            page_height = page_width * (
                rows / cols
            )  # Make height proportional to maintain square buttons
            button_size = min(page_width / cols, page_height / rows)  # Square buttons

            # Convert buttons
            for btn in page.buttons:
                # Use absolute position if available, otherwise calculate from grid
                if btn.left is not None and btn.top is not None:
                    x = int(btn.left * page_width)
                    y = int(btn.top * page_height)
                else:
                    row, col = btn.position
                    x = int(col * button_size)
                    y = int(row * button_size)

                # Use square button dimensions
                width = int(button_size)
                height = int(button_size)

                button = {
                    "ButtonType": 0,
                    "DisplayColor": self._convert_hex_to_apple_color(
                        btn.style.body_color or "#ffffff"
                    ),
                    "DisplayImageResourceIsTemplate": False,
                    "DisplayImageWeight": "FontWeightRegular",
                    "DisplayText": btn.label,
                    "FontSize": 12,
                    "ID": btn.id,
                    "PanelObjectType": "Button",
                    "Rect": f"{{{{{x}, {y}}}, {{{width}, {height}}}}}",
                }

                # Handle image if present
                if btn.image and btn.image.get("url"):
                    import uuid

                    # Generate unique image ID in Apple format
                    image_id = f"Image.{str(uuid.uuid4()).upper()}"

                    # Add to assets index with proper format
                    assets[image_id] = {
                        "Type": "Image",
                        "Name": btn.label
                        or "Button Image",  # Use button label as image name
                    }

                    # Add image reference to button
                    button["DisplayImageResource"] = image_id

                    # Download and save image directly in Resources
                    try:
                        import requests

                        response = requests.get(btn.image["url"])
                        if response.status_code == 200:
                            # Save image with the same ID as referenced
                            with open(resources_dir / image_id, "wb") as f:
                                f.write(response.content)
                    except Exception as e:
                        self.debug(f"Failed to download image {btn.image['url']}: {e}")

                # Add actions
                if btn.type == ButtonType.NAVIGATE and btn.target_page_id:
                    button["Actions"] = [
                        {
                            "ActionParam": {"PanelID": btn.target_page_id},
                            "ActionRecordedOffset": 0.0,
                            "ActionType": "ActionOpenPanel",
                            "ID": f"Action.{btn.id}",
                        }
                    ]
                elif btn.vocalization:
                    button["Actions"] = [
                        {
                            "ActionParam": {
                                "CharString": btn.vocalization,
                                "isStickyKey": False,
                            },
                            "ActionRecordedOffset": 0.0,
                            "ActionType": "ActionPressKeyCharSequence",
                            "ID": f"Action.{btn.id}",
                        }
                    ]

                panel["PanelObjects"].append(button)

            panels["Panels"][page.id] = panel

        # Save AssetIndex.plist
        with open(resources_dir / "AssetIndex.plist", "wb") as f:
            plistlib.dump(assets, f)

        # Save panel definitions
        with open(resources_dir / "PanelDefinitions.plist", "wb") as f:
            plistlib.dump(panels, f)

    def _convert_hex_to_apple_color(self, hex_color: str) -> str:
        """Convert hex color to Apple color string format.

        Args:
            hex_color: Color in hex format (#RRGGBB)

        Returns:
            Color string in format "r g b a" with values 0-1
        """
        try:
            # Remove # and convert to RGB
            rgb = tuple(int(hex_color.lstrip("#")[i : i + 2], 16) for i in (0, 2, 4))
            # Convert to 0-1 range and add alpha
            return f"{rgb[0]/255:.3f} {rgb[1]/255:.3f} {rgb[2]/255:.3f} 1.000"
        except (ValueError, IndexError):
            return "1.000 1.000 1.000 1.000"  # Default white

    def extract_texts(self, file_path: str) -> list[str]:
        """Extract all text content from Apple Panels config.

        Args:
            file_path: Path to .ascconfig folder

        Returns:
            List of extracted texts
        """
        tree = self.load_into_tree(file_path)
        texts = []

        for page in tree.pages.values():
            if page.name:
                texts.append(page.name)
            for btn in page.buttons:
                if btn.label:
                    texts.append(btn.label)
                if btn.vocalization and btn.vocalization != btn.label:
                    texts.append(btn.vocalization)

        return texts

    def process_files(
        self, directory: str, translations: Optional[dict[str, str]] = None
    ) -> Optional[str]:
        """Process files in directory.

        Args:
            directory: Directory containing files to process
            translations: Optional translations to apply

        Returns:
            Path to translated file if translations applied, None otherwise
        """
        try:
            # Load tree structure
            tree = self.load_into_tree(directory)

            if translations is None:
                # Just collect texts
                self.collected_texts = self.extract_texts(directory)
                return None

            # Apply translations
            for page in tree.pages.values():
                if page.name in translations:
                    page.name = translations[page.name]

                for btn in page.buttons:
                    if btn.label in translations:
                        btn.label = translations[btn.label]
                    if btn.vocalization in translations:
                        btn.vocalization = translations[btn.vocalization]

            # Save translated version
            target_lang = translations.get("target_lang", "unknown")
            base = os.path.splitext(directory)[0]
            output_path = f"{base}_{target_lang}.ascconfig"
            self.save_from_tree(tree, output_path)

            return output_path

        except Exception as e:
            self.debug(f"Error processing files: {str(e)}")
            return None

    def create_translated_file(
        self, file_path: str, translations: dict[str, str]
    ) -> Optional[str]:
        """Create a translated version of the file.

        Args:
            file_path: Path to source file
            translations: Dictionary of text translations

        Returns:
            Path to translated file if successful, None otherwise
        """
        try:
            # Load tree structure
            tree = self.load_into_tree(file_path)

            # Apply translations
            for page in tree.pages.values():
                if page.name in translations:
                    page.name = translations[page.name]

                for btn in page.buttons:
                    if btn.label in translations:
                        btn.label = translations[btn.label]
                    if btn.vocalization in translations:
                        btn.vocalization = translations[btn.vocalization]

            # Save translated version
            target_lang = translations.get("target_lang", "unknown")
            base = os.path.splitext(file_path)[0]
            output_path = f"{base}_{target_lang}.ascconfig"
            self.save_from_tree(tree, output_path)

            return output_path

        except Exception as e:
            self.debug(f"Error creating translated file: {str(e)}")
            return None



import os
import shutil
import tempfile
import zipfile
from abc import abstractmethod
from collections.abc import Iterator
from typing import Any, Optional, Union

from .base_processor import AACProcessor
from .tree_structure import AACTree


class FileProcessor(AACProcessor):
    """Base class for AAC file processors that work with files."""

    def __init__(self) -> None:
        """Initialize the file processor."""
        super().__init__()
        self.file_path: Optional[str] = None
        self.original_filename: Optional[str] = None
        self.original_file_path: Optional[str] = None
        self._temp_dirs: list[str] = []
        self.collected_texts: list[str] = []
        self._debug_output = print  # Default debug output

    def set_source_file(self, file_path: str) -> None:
        """Set source file path.

        Args:
            file_path: Path to source file.
        """
        self.file_path = file_path
        self.original_file_path = file_path
        self.original_filename = os.path.splitext(os.path.basename(file_path))[0]

    def create_temp_dir(self) -> str:
        """Create a temporary directory.

        Returns:
            Path to created temporary directory.
        """
        temp_dir = tempfile.mkdtemp()
        self._temp_dirs.append(temp_dir)
        return temp_dir

    def cleanup_temp_files(self) -> None:
        """Clean up temporary files."""
        for temp_dir in self._temp_dirs:
            if os.path.exists(temp_dir):
                shutil.rmtree(temp_dir)
        self._temp_dirs = []

    def _prepare_workspace(self, file_path: str) -> str:
        """Prepare workspace for processing.

        Args:
            file_path: Path to file to process.

        Returns:
            Path to workspace directory.
        """
        workspace = self.create_temp_dir()
        self.set_source_file(file_path)
        return workspace

    @abstractmethod
    def process_files(
        self, directory: str, translations: Optional[dict[str, str]] = None
    ) -> Optional[str]:
        """Process files in directory.

        Args:
            directory: Directory containing files to process.
            translations: Dictionary of translations.

        Returns:
            Path to translated file if successful, None otherwise.
        """
        pass

    def analyze_vocabulary(self, tree: AACTree) -> dict[str, Any]:
        """Analyze vocabulary in tree structure.

        Args:
            tree: Tree structure to analyze.

        Returns:
            Dictionary containing vocabulary analysis.
        """
        analysis: dict[str, Any] = {
            "total_buttons": 0,
            "unique_words": set[str](),
            "word_frequency": dict[str, int](),
            "button_types": {
                "speak": 0,
                "navigate": 0,
                "action": 0,
                "wordlist": 0,
                "command": 0,
            },
        }

        for page in tree.pages.values():
            for button in page.buttons:
                analysis["total_buttons"] += 1
                analysis["button_types"][button.type.value] += 1

                if button.vocalization:
                    words = button.vocalization.lower().split()
                    for word in words:
                        analysis["unique_words"].add(word)
                        analysis["word_frequency"][word] = (
                            analysis["word_frequency"].get(word, 0) + 1
                        )

        # Convert set to list for JSON serialization
        analysis["unique_words"] = list(analysis["unique_words"])
        return analysis

    def _walk_files(self, directory: str) -> Iterator[tuple[str, str]]:
        """Walk through files in directory.

        Args:
            directory: Directory to walk through.

        Yields:
            Tuple of (relative path, absolute path) for each file.
        """
        for root, _, files in os.walk(directory):
            for file in files:
                abs_path = os.path.join(root, file)
                rel_path = os.path.relpath(abs_path, directory)
                yield rel_path, abs_path

    def _copy_file(self, src: str, dst: str) -> None:
        """Copy file with proper error handling.

        Args:
            src: Source file path.
            dst: Destination file path.
        """
        try:
            os.makedirs(os.path.dirname(dst), exist_ok=True)
            shutil.copy2(src, dst)
        except Exception as e:
            self.debug(f"Error copying file {src} to {dst}: {str(e)}")
            raise

    def _sanitize_name(self, name: Optional[str]) -> str:
        """Sanitize a filename.

        Args:
            name: Name to sanitize.

        Returns:
            Sanitized name.
        """
        if not name:
            return ""
        # Remove punctuation and convert spaces/hyphens to underscores
        import re

        sanitized = re.sub(r"[^\w\s-]", "", name)
        sanitized = re.sub(r"[-\s]+", "_", sanitized)
        return sanitized.lower().strip("_")

    def get_output_path(self, target_lang: Optional[str] = None) -> str:
        """Get output path for translated file.

        Args:
            target_lang: Target language code.

        Returns:
            Path for translated file.

        Raises:
            ValueError: If no source file has been set.
        """
        if not self.file_path:
            raise ValueError("No source file has been set")

        if not target_lang:
            target_lang = "translated"

        # Get base name without any existing language suffix
        if self.original_filename:
            base_name = self.original_filename
        else:
            base_name = os.path.splitext(os.path.basename(self.file_path))[0]

        if "_" in base_name:
            base_parts = base_name.split("_")
            if len(base_parts[-1]) <= 5:  # Assuming language codes are <= 5 chars
                base_name = "_".join(base_parts[:-1])

        # Get original extension
        ext = os.path.splitext(self.file_path)[1]

        # Create output name with target language
        output_name = f"{base_name}_{target_lang}{ext}"
        return os.path.join(os.path.dirname(self.file_path), output_name)

    def process_texts(
        self,
        file_path: str,
        translations: Optional[dict[str, str]] = None,
        output_path: Optional[str] = None,
    ) -> Union[list[str], str, None]:
        """Process texts in file.

        Args:
            file_path: Path to file to process.
            translations: Dictionary of translations.
            output_path: Optional path where to save translated file.

        Returns:
            Union[List[str], str, None]: List of texts if extracting,
            path to translated file if translating, None if error.
        """
        try:
            # Reset state for new translation
            self.collected_texts = []
            self.set_source_file(file_path)

            # Create temp directory for processing
            temp_dir = self.create_temp_dir()

            # Extract archive if necessary, or copy file
            if self.check_is_archive(file_path):
                self.extract_archive(file_path, temp_dir)
            else:
                # Copy file to temp directory
                temp_file = os.path.join(temp_dir, os.path.basename(file_path))
                shutil.copy2(file_path, temp_file)

            # Process the files
            result = self.process_files(temp_dir, translations)

            if translations is None:
                # In extract mode, return extracted texts
                return self.extract_texts(file_path)

            if result:
                if output_path:
                    # Copy result to output path
                    shutil.copy2(result, output_path)
                    return output_path
                return result

            return None

        except Exception as e:
            self.debug(f"Error processing texts: {str(e)}")
            return None
        finally:
            self.cleanup_temp_files()

    def check_is_archive(self, file_path: Optional[str]) -> bool:
        """Check if file is an archive.

        Args:
            file_path: Path to file to check.

        Returns:
            bool: True if file is an archive.
        """
        if not file_path:
            return False
        try:
            with zipfile.ZipFile(file_path, "r") as _:
                return True
        except zipfile.BadZipFile:
            return False

    def extract_archive(self, archive_path: str, target_dir: str) -> None:
        """Extract archive to target directory.

        Args:
            archive_path: Path to archive file.
            target_dir: Directory to extract to.
        """
        with zipfile.ZipFile(archive_path, "r") as zf:
            zf.extractall(target_dir)

    def create_archive(self, source_dir: str, output_path: str) -> None:
        """Create archive from directory.

        Args:
            source_dir: Directory to archive.
            output_path: Path where to save archive.
        """
        with zipfile.ZipFile(output_path, "w", zipfile.ZIP_DEFLATED) as zf:
            for root, _, files in os.walk(source_dir):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, source_dir)
                    zf.write(file_path, arcname)



#!/usr/bin/env python3

import argparse
import glob
import os
import readline
import sys
from typing import Optional, Union

from .coughdrop_processor import CoughDropProcessor
from .dot_processor import DotProcessor
from .gridset_processor import GridsetProcessor
from .opml_processor import OPMLProcessor
from .snap_processor import SnapProcessor
from .touchchat_processor import TouchChatProcessor
from .tree_structure import AACTree
from .viewer import get_processor_for_file, print_tree

# Type alias for processors
ProcessorType = Union[
    GridsetProcessor,
    TouchChatProcessor,
    SnapProcessor,
    CoughDropProcessor,
    OPMLProcessor,
    DotProcessor,
]


def complete_path(text: str, state: int) -> Optional[str]:
    """Tab completion function for file paths"""
    if "~" in text:
        text = os.path.expanduser(text)

    # If text is a directory, return it with a trailing slash
    if os.path.isdir(text):
        return text + "/" if not text.endswith("/") else text

    dir_name = os.path.dirname(text)
    if dir_name and not os.path.exists(dir_name):
        return None

    # Get pattern for matching
    if dir_name:
        pattern = os.path.join(dir_name, os.path.basename(text) + "*")
    else:
        pattern = text + "*"

    # Get matches and sort them (directories first)
    matches = glob.glob(pattern)
    matches = sorted(
        [f"{m}{'/' if os.path.isdir(m) else ''}" for m in matches],
        key=lambda x: (not x.endswith("/"), x),  # Sort directories first
    )

    return matches[state] if state < len(matches) else None


def get_available_formats() -> list[str]:
    """Get list of available format names"""
    return ["grid", "touchchat", "snap", "coughdrop", "opml", "dot"]


def convert_format(
    input_file: str, output_format: str, output_path: Optional[str] = None
) -> Optional[str]:
    """Convert between AAC formats

    Args:
        input_file: Path to input AAC file
        output_format: Target format name
        output_path: Optional output path (default: auto-generated)

    Returns:
        Path to output file if successful, None otherwise
    """
    try:
        # Get source processor
        source_processor = get_processor_for_file(input_file)
        if not source_processor:
            print(f"Error: Unsupported input format: {input_file}")
            return None

        print(f"Loading {input_file}")
        # Load into tree structure
        tree = source_processor.load_into_tree(input_file)
        if not tree or not isinstance(tree, AACTree):
            print("Error: Failed to load input file into tree structure")
            return None

        print(f"Loaded tree with {len(tree.pages)} pages")

        # Get target processor based on format
        target_processor: Optional[ProcessorType] = None
        if output_format == "grid":
            target_processor = GridsetProcessor()
        elif output_format == "touchchat":
            target_processor = TouchChatProcessor()
        elif output_format == "snap":
            target_processor = SnapProcessor()
        elif output_format == "coughdrop":
            target_processor = CoughDropProcessor()
        elif output_format == "opml":
            target_processor = OPMLProcessor()
        elif output_format == "dot":
            target_processor = DotProcessor()

        if not target_processor:
            print(f"Error: Unsupported output format: {output_format}")
            return None

        # Generate output path if not provided
        if not output_path:
            base = os.path.splitext(input_file)[0]
            # Each processor should define its default extension
            ext = getattr(target_processor, "default_extension", ".unknown")
            output_path = f"{base}_converted{ext}"

        print(f"Converting to {output_format} format at {output_path}")
        # Export tree using target processor
        target_processor.export_tree(tree, output_path)
        return output_path

    except Exception as e:
        print(f"Error during conversion: {str(e)}")
        return None


def interactive_mode() -> None:
    """Interactive CLI mode"""
    print("\nAAC Processor Tool")
    print("=================\n")

    # Set up tab completion
    readline.set_completer_delims(" \t\n;")
    readline.parse_and_bind("tab: complete")
    readline.set_completer(complete_path)

    # Get input file
    while True:
        file_path = input("Enter path to AAC file: ").strip()
        file_path = os.path.expanduser(file_path)
        if os.path.exists(file_path):
            break
        print("Error: File not found. Please try again.")

    # Reset completer
    readline.set_completer(None)

    processor = get_processor_for_file(file_path)
    if not processor:
        print(f"Error: Unsupported file type: {file_path}")
        sys.exit(1)

    # Show options
    print("\nAvailable actions:")
    print("1. View AAC structure")
    print("2. Convert to different format")
    print("3. Exit")

    while True:
        choice = input("\nSelect action (1-3): ").strip()

        if choice == "1":
            tree = processor.load_into_tree(file_path)
            print_tree(tree)
            break

        elif choice == "2":
            print("\nAvailable formats:")
            formats = get_available_formats()
            for i, fmt in enumerate(formats, 1):
                print(f"{i}. {fmt}")

            while True:
                fmt_choice = input(
                    f"\nSelect target format (1-{len(formats)}): "
                ).strip()
                try:
                    target_format = formats[int(fmt_choice) - 1]
                    break
                except (ValueError, IndexError):
                    print("Invalid choice. Please try again.")

            prompt = "Enter output path (or press Enter for default): "
            output_path: Optional[str] = input(prompt).strip()
            output_path = output_path if output_path else None
            if convert_format(file_path, target_format, output_path):
                print("Conversion successful")
            break

        elif choice == "3":
            print("Goodbye!")
            sys.exit(0)

        else:
            print("Invalid choice. Please try again.")


def main() -> None:
    """Main entry point for the CLI"""
    parser = argparse.ArgumentParser(description="AAC Format Processor and Converter")

    subparsers = parser.add_subparsers(dest="command", help="Command to execute")

    # View command
    view_parser = subparsers.add_parser("view", help="View AAC structure")
    view_parser.add_argument("file", help="Path to AAC file")

    # Convert command
    convert_parser = subparsers.add_parser("convert", help="Convert AAC format")
    convert_parser.add_argument("input", help="Input AAC file")
    convert_parser.add_argument(
        "--to", choices=get_available_formats(), required=True, help="Target format"
    )
    convert_parser.add_argument("--output", help="Output file path (optional)")

    args = parser.parse_args()

    # If no command specified, run interactive mode
    if not args.command:
        interactive_mode()
        return

    # Handle view command
    if args.command == "view":
        if not os.path.exists(args.file):
            print(f"Error: File not found: {args.file}")
            sys.exit(1)

        processor = get_processor_for_file(args.file)
        if not processor:
            print(f"Error: Unsupported file type: {args.file}")
            sys.exit(1)

        try:
            tree = processor.load_into_tree(args.file)
            print_tree(tree)
        except Exception as e:
            print(f"Error viewing file: {str(e)}")
            sys.exit(1)

    # Handle convert command
    elif args.command == "convert":
        if not os.path.exists(args.input):
            print(f"Error: File not found: {args.input}")
            sys.exit(1)

        if convert_format(args.input, args.to, args.output):
            print("Conversion successful")
        else:
            print("Conversion failed")
            sys.exit(1)


if __name__ == "__main__":
    main()



import logging
import os
import zipfile
from typing import Optional, cast

from lxml import etree
from lxml.etree import _Element, _ElementTree

from .file_processor import FileProcessor
from .tree_structure import AACButton, AACPage, AACTree, ButtonType

# Set up logging
logger = logging.getLogger(__name__)


class GridsetProcessor(FileProcessor):
    """Processor for Grid3 files (.gridset)."""

    def __init__(self) -> None:
        """Initialize the GridsetProcessor."""
        super().__init__()
        self.collected_texts = []

    def can_process(self, file_path: str) -> bool:
        """Check if file is a Grid3 gridset.

        Args:
            file_path (str): Path to the file to check.

        Returns:
            bool: True if file is a Grid3 gridset.
        """
        return file_path.lower().endswith(".gridset")

    def process_files(
        self, directory: str, translations: Optional[dict[str, str]] = None
    ) -> Optional[str]:
        """Process XML files in the gridset directory.

        Args:
            directory (str): Directory containing the files to process.
            translations (Optional[Dict[str, str]]): Dictionary of translations.

        Returns:
            Optional[str]: Path to translated file if successful, None otherwise.
        """
        modified = False
        self.collected_texts = []

        # Look for Grids directory
        grids_dir = os.path.join(directory, "Grids")
        if not os.path.exists(grids_dir):
            self.debug("No Grids directory found")
            return None

        # Walk through all subdirectories in Grids
        for root, _, files in os.walk(grids_dir):
            for file in files:
                if file == "grid.xml":
                    grid_path = os.path.join(root, file)
                    try:
                        parser = etree.XMLParser(remove_blank_text=True)
                        tree = etree.parse(grid_path, parser)
                        grid_root = tree.getroot()

                        # Process all captions
                        for caption in grid_root.xpath(".//Caption"):
                            if caption.text and caption.text.strip():
                                text = caption.text.strip()
                                if translations is None:
                                    self.collected_texts.append(text)
                                elif text in translations:
                                    caption.text = translations[text]
                                    modified = True

                        # Process all text parameters
                        for element in grid_root.xpath(".//Parameter[@Key='text']"):
                            try:
                                full_text, metadata, original_parts = (
                                    self._extract_text_and_metadata_from_element(
                                        element
                                    )
                                )
                                if full_text.strip():
                                    if translations is None:
                                        self.collected_texts.append(full_text)
                                    elif full_text in translations:
                                        translated_text = translations[full_text]
                                        self._update_element_with_translation(
                                            element,
                                            translated_text,
                                            metadata,
                                            original_parts,
                                        )
                                        modified = True
                            except Exception as e:
                                self.debug(f"Error processing text parameter: {str(e)}")
                                continue

                        # Process WordList items
                        for text_elem in grid_root.xpath(".//WordListItem/Text//r"):
                            if text_elem.text and text_elem.text.strip():
                                text = text_elem.text.strip()
                                if translations is None:
                                    self.collected_texts.append(text)
                                elif text in translations:
                                    text_elem.text = translations[text]
                                    modified = True

                        if modified:
                            tree.write(
                                grid_path,
                                encoding="utf-8",
                                xml_declaration=True,
                                pretty_print=True,
                            )
                            self.debug(f"Saved changes to {grid_path}")

                    except Exception as e:
                        self.debug(f"Error processing grid file {grid_path}: {str(e)}")
                        continue

        # If translations were applied, create new gridset
        if modified and translations is not None:
            # Get target language from translations dict
            target_lang = translations.get("target_lang", "unknown")

            # Create new filename with target language
            base_name = os.path.splitext(directory)[0]
            translated_path = f"{base_name}_{target_lang}.gridset"

            # Create new gridset file
            with zipfile.ZipFile(translated_path, "w", zipfile.ZIP_DEFLATED) as zip_ref:
                # Add all files from temp directory to the new gridset
                for root, _, files in os.walk(directory):
                    for file in files:
                        file_path = os.path.join(root, file)
                        arcname = os.path.relpath(file_path, directory)
                        zip_ref.write(file_path, arcname)
                        logger.debug(f"Added to zip: {arcname}")

            logger.debug(f"Created translated gridset: {translated_path}")
            return translated_path  # Return the path to the translated file

        return None

    def _extract_text_and_metadata_from_element(
        self, element: etree.Element
    ) -> tuple[str, list[dict], list[tuple[str, bool]]]:
        """Extract text and metadata from an element.

        Args:
            element (etree.Element): XML element to process.

        Returns:
            Tuple[str, List[Dict], List[Tuple[str, bool]]]:
                Text, metadata, and text parts.
        """
        text_parts = []
        metadata = []
        for s_elem in element.findall(".//s"):
            s_meta = dict(s_elem.attrib) if s_elem.attrib else {}
            for r_elem in s_elem.findall(".//r"):
                if r_elem.text:
                    # Store whether this was originally a CDATA space
                    is_space = r_elem.text.isspace()
                    text = r_elem.text.strip()
                    if text or is_space:  # Include empty strings that were spaces
                        text_parts.append((text, is_space))
            metadata.append(s_meta)
        return (
            " ".join(part[0] for part in text_parts if part[0]),
            metadata,
            text_parts,
        )

    def _update_element_with_translation(
        self,
        element: etree.Element,
        translated_text: str,
        metadata: list[dict],
        original_parts: Optional[list[tuple[str, bool]]] = None,
    ) -> None:
        """Update an element with translated text and preserve metadata.

        Args:
            element (etree.Element): XML element to update.
            translated_text (str): Translated text to insert.
            metadata (List[Dict]): List of metadata dictionaries.
            original_parts (Optional[List[Tuple[str, bool]]]): Original text parts.
        """
        # Clear existing children
        for child in element:
            element.remove(child)

        # Rebuild the structure with translated text
        p_elem = etree.SubElement(element, "p")

        # Split the translated text while preserving spaces
        words = []
        current_word = ""
        for char in translated_text:
            if char.isspace():
                if current_word:
                    words.append((current_word, False))  # Regular word
                    words.append((" ", True))  # Space
                    current_word = ""
                elif not words or words[-1][0] != " ":
                    words.append((" ", True))  # Space
            else:
                current_word += char
        if current_word:
            words.append((current_word, False))

        # Create elements for each word/space
        for i, (word, is_space) in enumerate(words):
            s_elem = etree.SubElement(p_elem, "s")
            if i < len(metadata):
                for k, v in metadata[i].items():
                    s_elem.set(k, v)
            r_elem = etree.SubElement(s_elem, "r")
            if is_space:
                r_elem.text = etree.CDATA(" ")
            else:
                r_elem.text = word

    def _create_cdata(self, text: str) -> etree.CDATA:
        """Create a CDATA section.

        Args:
            text (str): Text to wrap in CDATA.

        Returns:
            etree.CDATA: CDATA section.
        """
        return etree.CDATA(text)

    def extract_texts(self, file_path: str) -> list[str]:
        """Extract translatable texts from gridset.

        Args:
            file_path (str): Path to the gridset file.

        Returns:
            List[str]: List of translatable texts.
        """
        texts = []
        temp_dir = self.create_temp_dir()

        with zipfile.ZipFile(file_path, "r") as zf:
            zf.extractall(temp_dir)

            # Process each grid.xml file
            grids_dir = os.path.join(temp_dir, "Grids")
            if os.path.exists(grids_dir):
                for grid_dir in os.listdir(grids_dir):
                    grid_path = os.path.join(grids_dir, grid_dir, "grid.xml")
                    if not os.path.exists(grid_path):
                        continue

                    try:
                        tree = etree.parse(grid_path)
                        root = tree.getroot()

                        # Extract grid name
                        name = root.get("Name")
                        if name:
                            texts.append(name)

                        # Extract captions
                        for caption in root.findall(".//CaptionAndImage/Caption"):
                            if caption.text and caption.text.strip():
                                texts.append(caption.text.strip())

                        # Extract wordlist items
                        for text in root.findall(".//WordList/Items/WordListItem/Text"):
                            if text.text and text.text.strip():
                                texts.append(text.text.strip())

                    except Exception as e:
                        self.debug(f"Error processing grid file {grid_path}: {str(e)}")

        return list(set(texts))  # Remove duplicates

    def load_into_tree(self, file_path: str) -> AACTree:
        """Load gridset into tree structure.

        Args:
            file_path (str): Path to the gridset file.

        Returns:
            AACTree: Tree structure representing the gridset.
        """
        tree = AACTree()
        self.tree = tree  # Store reference to tree

        # Extract gridset
        temp_dir = self.create_temp_dir()
        with zipfile.ZipFile(file_path, "r") as zf:
            zf.extractall(temp_dir)

            # Find all grid.xml files
            grids_dir = os.path.join(temp_dir, "Grids")
            if not os.path.exists(grids_dir):
                self.debug("No Grids directory found")
                return tree

            # Process each grid.xml file
            for grid_dir in os.listdir(grids_dir):
                grid_path = os.path.join(grids_dir, grid_dir, "grid.xml")
                if not os.path.exists(grid_path):
                    continue

                try:
                    grid_tree = etree.parse(grid_path)
                    grid_root = grid_tree.getroot()

                    # Get grid dimensions
                    rows = len(grid_root.findall(".//RowDefinitions/RowDefinition"))
                    cols = len(
                        grid_root.findall(".//ColumnDefinitions/ColumnDefinition")
                    )

                    # Create page
                    page = AACPage(
                        id=grid_dir, name=grid_dir, grid_size=(rows or 1, cols or 1)
                    )

                    # Process cells
                    for cell in grid_root.findall(".//Cells/Cell"):
                        x = int(cell.get("X", "0"))
                        y = int(cell.get("Y", "0"))

                        # Get content
                        content = cell.find(".//Content")
                        if content is None:
                            continue

                        # Get caption
                        caption = content.find(".//CaptionAndImage/Caption")
                        if caption is not None and caption.text:
                            # Create button
                            button = AACButton(
                                id=f"{grid_dir}_button_{x}_{y}",
                                label=caption.text.strip(),
                                type=ButtonType.SPEAK,
                                position=(x, y),
                                vocalization=caption.text.strip(),
                            )

                            # Check for navigation
                            commands = content.findall(".//Commands/Command")
                            for command in commands:
                                if command.get("ID") == "Jump.To":
                                    target = command.find(".//Parameter[@Key='grid']")
                                    if target is not None and target.text:
                                        button.type = ButtonType.NAVIGATE
                                        button.target_page_id = target.text

                            page.buttons.append(button)

                    # Process wordlist items if any
                    for item in grid_root.findall(".//WordList/Items/WordListItem"):
                        text = item.find(".//Text")
                        if text is not None and text.text:
                            button = AACButton(
                                id=f"{grid_dir}_word_{text.text}",
                                label=text.text.strip(),
                                type=ButtonType.SPEAK,
                                position=(
                                    0,
                                    0,
                                ),  # Position doesn't matter for wordlists
                                vocalization=text.text.strip(),
                            )
                            page.buttons.append(button)

                    tree.add_page(page)

                except Exception as e:
                    self.debug(f"Error processing grid file {grid_path}: {str(e)}")
                    continue

            # Read settings to find start grid
            settings_path = os.path.join(temp_dir, "Settings0", "settings.xml")
            if os.path.exists(settings_path):
                try:
                    settings_tree = etree.parse(settings_path)
                    start_grid = settings_tree.find(".//StartGrid")
                    if start_grid is not None and start_grid.text:
                        tree.root_id = start_grid.text
                except Exception as e:
                    self.debug(f"Error reading settings: {str(e)}")

        return tree

    def save_from_tree(self, tree: AACTree, output_path: str) -> None:
        """Save tree structure back to gridset format.

        Args:
            tree (AACTree): Tree structure to save.
            output_path (str): Path where to save the file.
        """
        # Create temp directory for building gridset
        temp_dir = self.create_temp_dir()
        grids_dir = os.path.join(temp_dir, "Grids")
        os.makedirs(grids_dir)

        # Create settings directory
        settings_dir = os.path.join(temp_dir, "Settings0")
        os.makedirs(settings_dir)

        # Save settings
        settings = etree.Element("GridSetSettings")
        start_grid = etree.SubElement(settings, "StartGrid")
        start_grid.text = tree.root_id or next(iter(tree.pages.keys()))
        settings_path = os.path.join(settings_dir, "settings.xml")
        etree.ElementTree(settings).write(
            settings_path, encoding="utf-8", xml_declaration=True, pretty_print=True
        )

        # Process each page
        for page_id, page in tree.pages.items():
            # Create grid directory
            grid_dir = os.path.join(grids_dir, page_id)
            os.makedirs(grid_dir)

            # Create grid XML
            grid = etree.Element("Grid")
            grid.set("Name", page.name)
            grid.set("GridGuid", page.id)

            # Add row and column definitions
            row_defs = etree.SubElement(grid, "RowDefinitions")
            for _ in range(page.grid_size[0]):
                etree.SubElement(row_defs, "RowDefinition")

            col_defs = etree.SubElement(grid, "ColumnDefinitions")
            for _ in range(page.grid_size[1]):
                etree.SubElement(col_defs, "ColumnDefinition")

            # Add cells
            cells = etree.SubElement(grid, "Cells")
            for button in page.buttons:
                # Skip wordlist items - they go in the WordList section
                if button.id.startswith(f"{page_id}_word_"):
                    continue

                cell = etree.SubElement(cells, "Cell")
                cell.set("X", str(button.position[0]))
                cell.set("Y", str(button.position[1]))

                content = etree.SubElement(cell, "Content")
                caption_and_image = etree.SubElement(content, "CaptionAndImage")
                caption = etree.SubElement(caption_and_image, "Caption")
                caption.text = button.label

                if button.type == ButtonType.NAVIGATE:
                    commands = etree.SubElement(content, "Commands")
                    command = etree.SubElement(commands, "Command")
                    command.set("ID", "Jump.To")
                    param = etree.SubElement(command, "Parameter")
                    param.set("Key", "grid")
                    param.text = button.target_page_id

            # Add wordlist items
            wordlist_buttons = [
                b for b in page.buttons if b.id.startswith(f"{page_id}_word_")
            ]
            if wordlist_buttons:
                wordlist = etree.SubElement(grid, "WordList")
                wordlist.set("Name", page.name)
                items = etree.SubElement(wordlist, "Items")
                for button in wordlist_buttons:
                    item = etree.SubElement(items, "WordListItem")
                    text = etree.SubElement(item, "Text")
                    text.text = button.label

            # Save grid XML
            grid_path = os.path.join(grid_dir, "grid.xml")
            etree.ElementTree(grid).write(
                grid_path, encoding="utf-8", xml_declaration=True, pretty_print=True
            )

        # Create FileMap.xml
        filemap = etree.Element("FileMap")
        entries = etree.SubElement(filemap, "Entries")
        for page_id in tree.pages:
            entry = etree.SubElement(entries, "Entry")
            entry.set("StaticFile", f"Grids\\{page_id}\\grid.xml")

        filemap_path = os.path.join(temp_dir, "FileMap.xml")
        etree.ElementTree(filemap).write(
            filemap_path, encoding="utf-8", xml_declaration=True, pretty_print=True
        )

        # Create gridset file
        with zipfile.ZipFile(output_path, "w", zipfile.ZIP_DEFLATED) as zip_ref:
            for root, _, files in os.walk(temp_dir):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, temp_dir)
                    zip_ref.write(file_path, arcname)

        self.debug(f"Created gridset file: {output_path}")

    def export_tree(self, tree: AACTree, output_path: str) -> None:
        """Export tree to GridSet format.

        Args:
            tree (AACTree): Tree to export.
            output_path (str): Path where to save the file.
        """
        self.save_from_tree(tree, output_path)

    def create_translated_file(
        self, file_path: str, translations: dict[str, str]
    ) -> Optional[str]:
        """Create a translated version of the gridset.

        Args:
            file_path (str): Path to the original gridset file.
            translations (Dict[str, str]): Dictionary of translations.

        Returns:
            Optional[str]: Path to translated file if successful, None otherwise.
        """
        # Extract gridset
        temp_dir = self.create_temp_dir()
        with zipfile.ZipFile(file_path, "r") as zf:
            zf.extractall(temp_dir)

        modified = False
        grids_dir = os.path.join(temp_dir, "Grids")

        # Process each grid.xml file
        for grid_dir in os.listdir(grids_dir):
            grid_path = os.path.join(grids_dir, grid_dir, "grid.xml")
            if not os.path.exists(grid_path):
                continue

            try:
                tree = etree.parse(grid_path)
                root = tree.getroot()
                grid_modified = False

                # Translate grid name
                name = root.get("Name")
                if name in translations:
                    root.set("Name", translations[name])
                    grid_modified = True

                # Translate captions
                for caption in root.findall(".//CaptionAndImage/Caption"):
                    if caption.text and caption.text.strip() in translations:
                        caption.text = translations[caption.text.strip()]
                        grid_modified = True

                # Translate wordlist items
                for text in root.findall(".//WordList/Items/WordListItem/Text"):
                    if text.text and text.text.strip() in translations:
                        text.text = translations[text.text.strip()]
                        grid_modified = True

                if grid_modified:
                    tree.write(
                        grid_path,
                        encoding="utf-8",
                        xml_declaration=True,
                        pretty_print=True,
                    )
                    modified = True

            except Exception as e:
                self.debug(f"Error processing grid file {grid_path}: {str(e)}")

        if modified:
            # Get target language from translations
            target_lang = translations.get("target_lang")
            if not target_lang:
                self.debug("No target language found in translations")
                return None

            # Create base name without any existing language suffix
            base_name = os.path.splitext(file_path)[0]
            if "_" in base_name:
                base_parts = base_name.split("_")
                if len(base_parts[-1]) <= 5:  # Assuming language codes are <= 5 chars
                    base_name = "_".join(base_parts[:-1])

            # Create new filename with target language
            translated_path = f"{base_name}_{target_lang}.gridset"

            # Create new gridset file
            with zipfile.ZipFile(translated_path, "w", zipfile.ZIP_DEFLATED) as zip_ref:
                for root, _, files in os.walk(temp_dir):
                    for file in files:
                        file_path = os.path.join(root, file)
                        arcname = os.path.relpath(file_path, temp_dir)
                        zip_ref.write(file_path, arcname)

            return translated_path

        return None

    def _parse_grid_xml(self, grid_path: str) -> _Element:
        """Parse a grid XML file and return its root element"""
        tree: _ElementTree = etree.parse(grid_path)
        return tree.getroot()

    def _process_grid(self, grid_root: _Element) -> AACPage:
        """Process a grid XML element into an AACPage"""
        # Get grid name and ID
        name = cast(str, grid_root.get("Name", ""))
        grid_id = cast(str, grid_root.get("GridGuid", ""))

        # Get grid dimensions
        rows = len(grid_root.findall(".//RowDefinition"))
        cols = len(grid_root.findall(".//ColumnDefinition"))

        # Create page
        page = AACPage(id=grid_id, name=name, grid_size=(rows, cols))

        # Process cells
        for cell in grid_root.findall(".//Cell"):
            button = self._process_cell(cell)
            if button:
                page.buttons.append(button)

        return page

    def _process_cell(self, cell: _Element) -> Optional[AACButton]:
        """Process a cell XML element into an AACButton"""
        # Get content
        content = cell.find(".//Content")
        if content is None:
            return None

        # Get caption
        caption = content.find(".//CaptionAndImage/Caption")
        if caption is not None and caption.text:
            # Create button
            button = AACButton(
                id=f"{grid_dir}_button_{x}_{y}",
                label=caption.text.strip(),
                type=ButtonType.SPEAK,
                position=(x, y),
                vocalization=caption.text.strip(),
            )

            # Check for navigation
            commands = content.findall(".//Commands/Command")
            for command in commands:
                if command.get("ID") == "Jump.To":
                    target = command.find(".//Parameter[@Key='grid']")
                    if target is not None and target.text:
                        button.type = ButtonType.NAVIGATE
                        button.target_page_id = target.text

            return button

        return None

    def replace_cell_with_xml(
        self,
        gridset_path: str,
        target_caption: Optional[str],
        target_action: Optional[str],
        new_content_xml: str,
        output_path: str,
    ) -> None:
        """Replace a cell's content with a new XML fragment across the gridset.

        Args:
            gridset_path (str): Path to the original gridset file.
            target_caption (Optional[str]): Caption of the button to replace.
            target_action (Optional[str]): Action command of the button to replace.
            new_content_xml (str): New XML content for the cell.
            output_path (str): Path to save the modified gridset.
        """
        temp_dir = self.create_temp_dir()
        with zipfile.ZipFile(gridset_path, "r") as zf:
            zf.extractall(temp_dir)

        grids_dir = os.path.join(temp_dir, "Grids")
        if not os.path.exists(grids_dir):
            self.debug("No Grids directory found")
            return

        new_content_element = etree.fromstring(new_content_xml)

        for grid_dir in os.listdir(grids_dir):
            grid_path = os.path.join(grids_dir, grid_dir, "grid.xml")
            if not os.path.exists(grid_path):
                continue

            try:
                tree = etree.parse(grid_path)
                root = tree.getroot()
                grid_modified = False

                for cell in root.findall(".//Cell"):
                    content = cell.find(".//Content")
                    caption = (
                        content.find(".//CaptionAndImage/Caption")
                        if content is not None
                        else None
                    )
                    commands = (
                        content.find(".//Commands") if content is not None else None
                    )
                    action_command = (
                        commands.find(".//Command[@ID='Action.Speak']")
                        if commands is not None
                        else None
                    )

                    if (caption is not None and caption.text == target_caption) or (
                        action_command is not None and target_action == "Action.Speak"
                    ):
                        # Replace the cell content with new_content_element
                        cell.remove(content)
                        cell.append(new_content_element)
                        grid_modified = True

                if grid_modified:
                    tree.write(
                        grid_path,
                        encoding="utf-8",
                        xml_declaration=True,
                        pretty_print=True,
                    )

            except Exception as e:
                self.debug(f"Error processing grid file {grid_path}: {str(e)}")

        with zipfile.ZipFile(output_path, "w", zipfile.ZIP_DEFLATED) as zip_ref:
            for root, _, files in os.walk(temp_dir):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, temp_dir)
                    zip_ref.write(file_path, arcname)

        self.debug(f"Created modified gridset file: {output_path}")



import os
import sqlite3
import zipfile

from aac_processors.touchchat_processor import TouchChatProcessor
from aac_processors.tree_structure import ButtonType


def test_can_process():
    processor = TouchChatProcessor()
    assert processor.can_process("test.ce")
    assert not processor.can_process("test.txt")


def test_load_tree(test_touchchat_ce):
    processor = TouchChatProcessor()
    tree = processor.load_into_tree(test_touchchat_ce)

    assert len(tree.pages) == 1
    page = next(iter(tree.pages.values()))
    assert page.name == "Test Page"

    assert len(page.buttons) == 1
    button = page.buttons[0]
    assert button.label == "Test Button"
    assert button.vocalization == "Hello"
    assert button.type == ButtonType.SPEAK


def test_save_tree(test_touchchat_ce, temp_dir):
    processor = TouchChatProcessor()
    tree = processor.load_into_tree(test_touchchat_ce)

    # Save tree to new file
    output_path = os.path.join(temp_dir, "output.ce")
    processor.save_from_tree(tree, output_path)

    # Verify the saved file
    assert os.path.exists(output_path)

    # Extract and check database
    with zipfile.ZipFile(output_path, "r") as zip_ref:
        c4v_path = None
        for name in zip_ref.namelist():
            if name.endswith(".c4v"):
                c4v_path = name
                zip_ref.extract(name, temp_dir)
                break

        assert c4v_path is not None
        db_path = os.path.join(temp_dir, c4v_path)

        with sqlite3.connect(db_path) as conn:
            cursor = conn.cursor()
            cursor.execute(
                """
                SELECT b.label, b.message
                FROM buttons b
                JOIN resources r ON b.resource_id = r.id
                WHERE b.label = 'Test Button'
            """
            )
            row = cursor.fetchone()
            assert row is not None
            assert row[0] == "Test Button"
            assert row[1] == "Hello"


def test_translation(test_touchchat_ce, temp_dir):
    processor = TouchChatProcessor()

    print(f"Initial file: {test_touchchat_ce}")
    texts = processor.extract_texts(test_touchchat_ce)
    print(f"Extracted texts: {texts}")

    translations = {"Test Button": "Botón de prueba", "Hello": "Hola"}
    translated_file = processor.process_texts(test_touchchat_ce, translations)
    print(f"Translated file: {translated_file}")

    tree = processor.load_into_tree(translated_file)
    page = next(iter(tree.pages.values()))
    button = page.buttons[0]
    print(f"Button label after translation: {button.label}")


def test_check_is_archive(test_touchchat_ce):
    """Test check_is_archive method."""
    processor = TouchChatProcessor()
    assert processor.check_is_archive(test_touchchat_ce)
    assert not processor.check_is_archive("not_an_archive.txt")
    assert not processor.check_is_archive(None)


def test_extract_archive(test_touchchat_ce, temp_dir):
    """Test extract_archive method."""
    processor = TouchChatProcessor()
    output_dir = os.path.join(temp_dir, "extracted")
    os.makedirs(output_dir)
    processor.extract_archive(test_touchchat_ce, output_dir)
    assert os.path.exists(os.path.join(output_dir, "test.c4v"))


def test_process_workflow(test_touchchat_ce, temp_dir):
    """Test the complete workflow as it happens in app.py"""
    # Create a temp dir like app.py does
    import tempfile

    work_dir = tempfile.mkdtemp()

    # Initialize processor with the work dir
    processor = TouchChatProcessor()
    processor._temp_dirs = [work_dir]  # Use the same temp dir

    # Set up debug logging like app.py does
    import logging

    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    # Add a stream handler if none exists
    if not logger.handlers:
        handler = logging.StreamHandler()
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        )
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    processor._debug_output = logger.debug

    try:
        # Copy test file to work dir (like app.py does with uploaded file)
        import shutil

        test_file = os.path.join(work_dir, "test.ce")
        shutil.copy2(test_touchchat_ce, test_file)
        processor.debug(f"Copied test file from {test_touchchat_ce} to {test_file}")

        # Process texts (extraction phase)
        texts = processor.process_texts(test_file)
        assert texts is not None and len(texts) > 0, "No texts found to translate"

        # Verify expected texts are present
        assert "Test Button" in texts
        assert "Hello" in texts

        # Create translations
        translations = {
            "Test Button": "Botón de Prueba",
            "Hello": "Hola",
            "target_lang": "es",
        }

        # Process translations
        output_path = os.path.join(
            work_dir, f"{os.path.splitext(os.path.basename(test_file))[0]}_es.ce"
        )
        result = processor.process_texts(test_file, translations, output_path)
        assert result is not None, "Translation failed"

        # Verify the translated file exists and is not empty
        assert os.path.exists(result), "Translated file does not exist"
        assert os.path.getsize(result) > 0, "Translated file is empty"

    finally:
        # Clean up
        try:
            shutil.rmtree(work_dir)
        except FileNotFoundError:
            pass  # Directory might already be cleaned up



import json
import os
import shutil
import sqlite3
import tempfile
import zipfile
from collections.abc import Generator
from typing import Any

import pytest
from lxml import etree as et


@pytest.fixture
def temp_dir() -> Generator[str, None, None]:
    """Create a temporary directory for test files"""
    temp_dir = tempfile.mkdtemp()
    yield temp_dir
    # Cleanup
    if os.path.exists(temp_dir):
        shutil.rmtree(temp_dir)


@pytest.fixture
def test_snap_db(temp_dir: str) -> str:
    """Create a test Snap database"""
    db_path = os.path.join(temp_dir, "test.sps")
    conn = sqlite3.connect(db_path)
    cursor = conn.cursor()

    cursor.executescript(
        """
        CREATE TABLE Page (
            id INTEGER PRIMARY KEY,
            Title TEXT
        );

        CREATE TABLE Button (
            id INTEGER PRIMARY KEY,
            page_id INTEGER,
            Label TEXT,
            Message TEXT,
            position_x INTEGER,
            position_y INTEGER,
            FOREIGN KEY (page_id) REFERENCES Page(id)
        );

        CREATE TABLE ButtonAction (
            id INTEGER PRIMARY KEY,
            button_id INTEGER,
            action_type TEXT,
            target_page_id INTEGER,
            FOREIGN KEY (button_id) REFERENCES Button(id)
        );
    """
    )

    # Add test pages
    cursor.execute("INSERT INTO Page (id, Title) VALUES (1, 'Test Page')")
    cursor.execute("INSERT INTO Page (id, Title) VALUES (2, 'Second Page')")

    # Add test buttons
    cursor.execute(
        """
        INSERT INTO Button (id, page_id, Label, Message, position_x, position_y)
        VALUES (1, 1, 'Speak Button', 'Hello', 0, 0)
    """
    )
    cursor.execute(
        """
        INSERT INTO Button (id, page_id, Label, Message, position_x, position_y)
        VALUES (2, 1, 'Navigate', NULL, 1, 0)
    """
    )

    # Add button action
    cursor.execute(
        """
        INSERT INTO ButtonAction (button_id, action_type, target_page_id)
        VALUES (2, 'navigate', 2)
    """
    )

    conn.commit()
    conn.close()

    return db_path


@pytest.fixture
def test_gridset(temp_dir: str) -> str:
    """Create a test Grid3 gridset with realistic content"""
    gridset_dir = os.path.join(temp_dir, "test_gridset_dir")

    # Create Grids directory
    grids_dir = os.path.join(gridset_dir, "Grids")
    os.makedirs(grids_dir, exist_ok=True)

    # Create Settings0 directory
    settings_dir = os.path.join(gridset_dir, "Settings0")
    os.makedirs(settings_dir, exist_ok=True)

    # Create first grid directory and grid.xml
    grid1_dir = os.path.join(grids_dir, "Test Grid")
    os.makedirs(grid1_dir, exist_ok=True)

    grid1 = et.Element("Grid")
    grid1.set("Name", "Test Grid")
    grid1.set("GridGuid", "1")

    # Add row and column definitions
    row_defs = et.SubElement(grid1, "RowDefinitions")
    for _ in range(2):
        et.SubElement(row_defs, "RowDefinition")

    col_defs = et.SubElement(grid1, "ColumnDefinitions")
    for _ in range(2):
        et.SubElement(col_defs, "ColumnDefinition")

    # Add cells
    cells = et.SubElement(grid1, "Cells")
    cell = et.SubElement(cells, "Cell")
    cell.set("X", "0")
    cell.set("Y", "0")

    content = et.SubElement(cell, "Content")
    caption_and_image = et.SubElement(content, "CaptionAndImage")
    caption = et.SubElement(caption_and_image, "Caption")
    caption.text = "Test Button"

    grid1_path = os.path.join(grid1_dir, "grid.xml")
    et.ElementTree(grid1).write(grid1_path, encoding="utf-8", xml_declaration=True)

    # Create second grid directory (wordlist) and grid.xml
    grid2_dir = os.path.join(grids_dir, "Test List")
    os.makedirs(grid2_dir, exist_ok=True)

    grid2 = et.Element("Grid")
    grid2.set("Name", "Test List")
    grid2.set("GridGuid", "2")

    # Add row and column definitions
    row_defs = et.SubElement(grid2, "RowDefinitions")
    et.SubElement(row_defs, "RowDefinition")
    col_defs = et.SubElement(grid2, "ColumnDefinitions")
    et.SubElement(col_defs, "ColumnDefinition")

    # Add wordlist
    wordlist = et.SubElement(grid2, "WordList")
    wordlist.set("Name", "Test List")
    items = et.SubElement(wordlist, "Items")
    item = et.SubElement(items, "WordListItem")
    text = et.SubElement(item, "Text")
    text.text = "Test Word"

    grid2_path = os.path.join(grid2_dir, "grid.xml")
    et.ElementTree(grid2).write(grid2_path, encoding="utf-8", xml_declaration=True)

    # Create settings.xml
    settings = et.Element("GridSetSettings")
    start_grid = et.SubElement(settings, "StartGrid")
    start_grid.text = "Test Grid"

    settings_path = os.path.join(settings_dir, "settings.xml")
    et.ElementTree(settings).write(
        settings_path, encoding="utf-8", xml_declaration=True
    )

    # Create FileMap.xml
    filemap = et.Element("FileMap")
    entries = et.SubElement(filemap, "Entries")

    entry1 = et.SubElement(entries, "Entry")
    entry1.set("StaticFile", "Grids\\Test Grid\\grid.xml")

    entry2 = et.SubElement(entries, "Entry")
    entry2.set("StaticFile", "Grids\\Test List\\grid.xml")

    filemap_path = os.path.join(gridset_dir, "FileMap.xml")
    et.ElementTree(filemap).write(filemap_path, encoding="utf-8", xml_declaration=True)

    # Create gridset file
    zip_path = os.path.join(temp_dir, "test.gridset")
    with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zip_ref:
        for root, _, files in os.walk(gridset_dir):
            for file in files:
                file_path = os.path.join(root, file)
                arc_name = os.path.relpath(file_path, gridset_dir)
                zip_ref.write(file_path, arc_name)

    return zip_path


@pytest.fixture
def test_touchchat_ce(temp_dir: str) -> str:
    """Create a test TouchChat CE file"""
    db_path = os.path.join(temp_dir, "test.c4v")
    conn = sqlite3.connect(db_path)
    cursor = conn.cursor()

    cursor.executescript(
        """
        CREATE TABLE resources (
            id INTEGER PRIMARY KEY,
            rid TEXT,
            name TEXT,
            type INTEGER
        );

        CREATE TABLE pages (
            id INTEGER PRIMARY KEY,
            resource_id INTEGER,
            FOREIGN KEY (resource_id) REFERENCES resources(id)
        );

        CREATE TABLE buttons (
            id INTEGER PRIMARY KEY,
            resource_id INTEGER,
            label TEXT,
            message TEXT,
            page_id INTEGER,
            FOREIGN KEY (resource_id) REFERENCES resources(id),
            FOREIGN KEY (page_id) REFERENCES pages(id)
        );

        CREATE TABLE button_boxes (
            id INTEGER PRIMARY KEY,
            init_size_x INTEGER,
            init_size_y INTEGER
        );

        CREATE TABLE button_box_instances (
            id INTEGER PRIMARY KEY,
            button_box_id INTEGER,
            page_id INTEGER,
            FOREIGN KEY (button_box_id) REFERENCES button_boxes(id),
            FOREIGN KEY (page_id) REFERENCES pages(id)
        );

        CREATE TABLE button_box_cells (
            id INTEGER PRIMARY KEY,
            button_box_id INTEGER,
            resource_id INTEGER,
            location INTEGER,
            span_x INTEGER DEFAULT 1,
            span_y INTEGER DEFAULT 1,
            FOREIGN KEY (button_box_id) REFERENCES button_boxes(id),
            FOREIGN KEY (resource_id) REFERENCES resources(id)
        );

        CREATE TABLE actions (
            id INTEGER PRIMARY KEY,
            resource_id INTEGER,
            code INTEGER,
            FOREIGN KEY (resource_id) REFERENCES resources(id)
        );

        CREATE TABLE action_data (
            id INTEGER PRIMARY KEY,
            action_id INTEGER,
            key INTEGER,
            value TEXT,
            FOREIGN KEY (action_id) REFERENCES actions(id)
        );

        CREATE TABLE special_pages (
            id INTEGER PRIMARY KEY,
            name TEXT,
            page_id INTEGER,
            FOREIGN KEY (page_id) REFERENCES pages(id)
        );
    """
    )

    # Add test page
    cursor.execute(
        "INSERT INTO resources (rid, name, type) VALUES ('page1', 'Test Page', 1)"
    )
    page_resource_id = cursor.lastrowid
    cursor.execute(
        "INSERT INTO pages (id, resource_id) VALUES (1, ?)", (page_resource_id,)
    )

    # Create button box for the page
    cursor.execute("INSERT INTO button_boxes (init_size_x, init_size_y) VALUES (2, 2)")
    button_box_id = cursor.lastrowid

    # Link button box to page
    cursor.execute(
        """
        INSERT INTO button_box_instances (button_box_id, page_id)
        VALUES (?, 1)
        """,
        (button_box_id,),
    )

    # Add test button
    cursor.execute(
        "INSERT INTO resources (rid, name, type) VALUES ('btn1', 'Test Button', 2)"
    )
    btn_resource_id = cursor.lastrowid
    cursor.execute(
        """
        INSERT INTO buttons (resource_id, label, message, page_id)
        VALUES (?, 'Test Button', 'Hello', 1)
        """,
        (btn_resource_id,),
    )

    # Add button cell
    cursor.execute(
        """
        INSERT INTO button_box_cells (button_box_id, resource_id, location)
        VALUES (?, ?, 0)
        """,
        (button_box_id, btn_resource_id),
    )

    # Set home page
    cursor.execute(
        """
        INSERT INTO special_pages (name, page_id)
        VALUES ('Home', 1)
        """
    )

    conn.commit()
    conn.close()

    # Create CE file
    ce_path = os.path.join(temp_dir, "test.ce")
    with zipfile.ZipFile(ce_path, "w", zipfile.ZIP_DEFLATED) as zip_ref:
        zip_ref.write(db_path, "test.c4v")

    return ce_path


@pytest.fixture
def test_coughdrop_obf(temp_dir: str) -> str:
    """Create a test CoughDrop OBF file"""
    obf_path = os.path.join(temp_dir, "test.obf")

    board_data = {
        "format": "open-board-0.1",
        "id": "home",
        "name": "Test Board",
        "grid": {"rows": 2, "columns": 2, "order": [["btn1", "btn2"], [None, None]]},
        "buttons": [
            {"id": "btn1", "label": "Test Button", "vocalization": "Hello"},
            {"id": "btn2", "label": "Navigate", "load_board": {"id": "page2"}},
        ],
    }

    with open(obf_path, "w") as f:
        json.dump(board_data, f, indent=2)

    return obf_path


@pytest.fixture
def test_coughdrop_obz(temp_dir: str, test_coughdrop_obf: str) -> str:
    """Create a test CoughDrop OBZ file"""
    obz_path = os.path.join(temp_dir, "test.obz")
    with zipfile.ZipFile(obz_path, "w") as zipf:
        zipf.write(test_coughdrop_obf, os.path.basename(test_coughdrop_obf))
    return obz_path


@pytest.fixture
def test_dot_file(temp_dir: str) -> str:
    """Create a test DOT file"""
    dot_path = os.path.join(temp_dir, "test.dot")

    # Create a simple DOT file with a few nodes and edges
    dot_content = """digraph G {
        node1 [label="Home Page"];
        node2 [label="About"];
        node3 [label="Contact"];
        node4 [label="Products"];

        node1 -> node2 [label="Go to About"];
        node1 -> node3 [label="Go to Contact"];
        node1 -> node4 [label="View Products"];

        node2 -> node1 [label="Back to Home"];
        node3 -> node1 [label="Back to Home"];
        node4 -> node1 [label="Back to Home"];
    }
    """

    with open(dot_path, "w") as f:
        f.write(dot_content)

    return dot_path


@pytest.fixture
def test_opml_file(temp_dir: str) -> str:
    """Create a test OPML file"""
    opml_path = os.path.join(temp_dir, "test.opml")

    # Create a simple OPML file with a few outlines
    opml_content = """<?xml version="1.0" encoding="UTF-8"?>
    <opml version="2.0">
        <head>
            <title>Test OPML</title>
        </head>
        <body>
            <outline text="Main Page">
                <outline text="Category 1">
                    <outline text="Item 1" />
                    <outline text="Item 2" />
                </outline>
                <outline text="Category 2">
                    <outline text="Item 3" />
                    <outline text="Item 4" />
                </outline>
            </outline>
        </body>
    </opml>
    """

    with open(opml_path, "w") as f:
        f.write(opml_content)

    return opml_path


def pytest_configure(config: Any) -> None:
    """Register custom marks"""
    config.addinivalue_line("markers", "slow: mark test as slow to run")



import pytest

from aac_processors.tree_structure import (
    AACButton,
    AACPage,
    AACTree,
    ButtonStyle,
    ButtonType,
)


@pytest.fixture
def sample_button() -> AACButton:
    """Create a sample button for testing"""
    return AACButton(
        id="btn1",
        label="Test Button",
        type=ButtonType.SPEAK,
        position=(0, 0),
        vocalization="Hello",
        style=ButtonStyle(
            font_color="#000000",
            body_color="#FFFFFF",
            border_color="#000000",
            border_width=1,
        ),
    )


@pytest.fixture
def sample_page(sample_button: AACButton) -> AACPage:
    """Create a sample page with buttons"""
    page = AACPage(id="page1", name="Test Page", grid_size=(3, 3))
    page.buttons.extend(
        [
            sample_button,
            AACButton(
                "btn2", "Navigate", ButtonType.NAVIGATE, (0, 1), target_page_id="page2"
            ),
            AACButton(
                "page1_word_hello",
                "Hello",
                ButtonType.SPEAK,
                (0, 2),
                vocalization="Hello",
            ),
        ]
    )
    return page


def test_button_creation(sample_button: AACButton) -> None:
    """Test basic button creation and properties"""
    assert sample_button.id == "btn1"
    assert sample_button.label == "Test Button"
    assert sample_button.type == ButtonType.SPEAK
    assert sample_button.position == (0, 0)
    assert sample_button.vocalization == "Hello"
    assert sample_button.style.font_color == "#000000"


def test_page_creation() -> None:
    """Test page creation and button management"""
    page = AACPage(id="page1", name="Test Page", grid_size=(3, 3))

    button1 = AACButton("btn1", "Button 1", ButtonType.SPEAK, (0, 0))
    button2 = AACButton("btn2", "Button 2", ButtonType.NAVIGATE, (0, 1))
    wordlist_button = AACButton("page1_word_test", "Test", ButtonType.SPEAK, (0, 2))

    page.buttons.extend([button1, button2, wordlist_button])

    assert page.id == "page1"
    assert page.name == "Test Page"
    assert page.grid_size == (3, 3)
    assert len(page.buttons) == 3
    assert page.buttons[0].type == ButtonType.SPEAK
    assert page.buttons[1].type == ButtonType.NAVIGATE
    # Verify wordlist button
    assert page.buttons[2].id.startswith("page1_word_")


def test_tree_navigation() -> None:
    """Test tree navigation and relationships"""
    tree = AACTree()

    # Create pages
    home = AACPage("home", "Home", (2, 2))
    page1 = AACPage("page1", "Page 1", (2, 2))
    page2 = AACPage("page2", "Page 2", (2, 2))

    # Set relationships
    page1.parent_id = "home"
    page2.parent_id = "home"

    # Add navigation buttons
    home.buttons.append(
        AACButton(
            "nav1", "To Page 1", ButtonType.NAVIGATE, (0, 0), target_page_id="page1"
        )
    )

    # Add pages to tree
    tree.add_page(home)
    tree.add_page(page1)
    tree.add_page(page2)

    # Test navigation
    assert tree.root_id == "home"
    assert len(tree.get_children("home")) == 2
    assert tree.get_path_to_page("page1") == ["home", "page1"]


def test_navigation_analysis() -> None:
    """Test navigation analysis functionality"""
    tree = AACTree()

    # Create a simple navigation structure
    home = AACPage("home", "Home", (2, 2))
    page1 = AACPage("page1", "Page 1", (2, 2))
    orphan = AACPage("orphan", "Orphan", (2, 2))

    # Add navigation button
    home.buttons.append(
        AACButton(
            "nav1", "To Page 1", ButtonType.NAVIGATE, (0, 0), target_page_id="page1"
        )
    )

    # Add pages to tree
    tree.add_page(home)
    tree.add_page(page1)
    tree.add_page(orphan)

    analysis = tree.analyze_navigation()

    assert analysis["total_pages"] == 3
    assert len(analysis["dead_ends"]) == 1  # Only page1 is a dead end
    assert len(analysis["orphaned_pages"]) == 1
    assert analysis["orphaned_pages"][0] == "orphan"



"""Example script showing how to convert an OBF file to Apple Panels format."""

from pathlib import Path

from aac_processors import viewer
from aac_processors.apple_panels_processor import ApplePanelsProcessor
from aac_processors.coughdrop_processor import CoughDropProcessor

# First install the package with dependencies:
# uv pip install -e ".[dev]"

# Initialize processors
coughdrop_processor = CoughDropProcessor()
panels_processor = ApplePanelsProcessor()

# Input OBF file
input_file = Path("examples/demofiles/project-core_es.obf")
output_file = input_file.with_suffix(".ascconfig")  # Apple Panels extension

# Load OBF into tree structure
tree = coughdrop_processor.load_into_tree(str(input_file))

# Show what we detected
print("\nDetected board structure:")
viewer.print_tree(tree)

# Print detailed page info
page = list(tree.pages.values())[0]  # First page
print("\nDetailed page information:")
print(f"Grid size: {page.grid_size}")
print(f"Found: {len(page.buttons)} buttons\n")

print("Buttons with text and images:")
for btn in page.buttons:
    has_image = bool(btn.image and btn.image.get("url"))
    image_info = f"(image: {btn.image['url']})" if has_image else "(no image)"
    print(f"- Position {btn.position}: '{btn.label}' {image_info}")

# Convert tree to Apple Panels format
panels_processor.save_from_tree(tree, str(output_file))

print(f"\nConverted {input_file} to {output_file}")
print("Images will be downloaded to Contents/Resources/Images/")



import os
from typing import Optional

import pytest

from aac_processors.optional.screenshot_processor import ScreenshotProcessor
from aac_processors.tree_structure import AACPage

# Mark for tests requiring screenshot dependencies
screenshot = pytest.mark.screenshot


@pytest.fixture
def demo_dir() -> str:
    """Get the demo files directory."""
    return "examples/demofiles"


@pytest.fixture
def screenshot_processor() -> ScreenshotProcessor:
    """Create a ScreenshotProcessor instance."""

    class TestProcessor(ScreenshotProcessor):
        def create_translated_file(
            self, file_path: str, translations: dict[str, str]
        ) -> Optional[str]:
            """Mock implementation of create_translated_file."""
            return "test_output.png"

    return TestProcessor()


@pytest.fixture
def demo_screenshots(demo_dir: str) -> list[str]:
    """Get paths to demo screenshot files."""
    files = [
        "TouchChat+HD+-+AAC+with+WordPower+60+Basic.jpg",
        "TouchChat24.png",
    ]
    return [os.path.join(demo_dir, f) for f in files]


@screenshot
def test_can_process(
    screenshot_processor: ScreenshotProcessor, demo_screenshots: list[str]
) -> None:
    """Test that processor can handle screenshot files."""
    for file_path in demo_screenshots:
        assert screenshot_processor.can_process(file_path)

    # Test invalid extensions
    assert not screenshot_processor.can_process("test.txt")
    assert not screenshot_processor.can_process("test.pdf")


@screenshot
def test_detect_grid(
    screenshot_processor: ScreenshotProcessor, demo_screenshots: list[str]
) -> None:
    """Test grid detection from screenshots."""
    for file_path in demo_screenshots:
        grid_width, grid_height, boxes = screenshot_processor.detect_grid(file_path)

        # Check grid dimensions are reasonable
        assert grid_width > 0
        assert grid_height > 0
        assert grid_width <= 20  # Reasonable max for AAC grids
        assert grid_height <= 20

        # Check we found some cells
        assert len(boxes) > 0

        # Check box coordinates are valid
        for x, y, w, h in boxes:
            assert x >= 0
            assert y >= 0
            assert w > 0
            assert h > 0


@screenshot
def test_detect_cell_content(
    screenshot_processor: ScreenshotProcessor, demo_screenshots: list[str]
) -> None:
    """Test content detection from grid cells."""
    for file_path in demo_screenshots:
        # Get grid info
        grid_width, grid_height, boxes = screenshot_processor.detect_grid(file_path)

        # Load image
        import cv2

        img = cv2.imread(file_path)
        assert img is not None

        # Test each cell
        for box in boxes:
            content = screenshot_processor.detect_cell_content(img, box)

            # Check content structure
            assert "text" in content
            assert "color" in content
            assert all(c in content["color"] for c in ["r", "g", "b"])

            # Check color values are valid
            for c in content["color"].values():
                assert 0 <= c <= 255


@screenshot
def test_create_page(
    screenshot_processor: ScreenshotProcessor, demo_screenshots: list[str]
) -> None:
    """Test creating AACPage from screenshots."""
    for file_path in demo_screenshots:
        page = screenshot_processor.create_page_from_screenshot(file_path)

        # Check page properties
        assert isinstance(page, AACPage)
        assert isinstance(page.id, str)
        assert page.id.startswith("screenshot_")
        assert isinstance(page.name, str)
        assert page.name.startswith("Detected Page")
        assert len(page.grid_size) == 2
        assert all(dim > 0 for dim in page.grid_size)

        # Check buttons
        assert len(page.buttons) > 0
        for btn in page.buttons:
            assert isinstance(btn.id, str)
            assert btn.id.startswith("btn_")
            assert len(btn.position) == 2
            assert all(pos >= 0 for pos in btn.position)
            assert btn.style is not None
            assert isinstance(btn.style.body_color, str)
            assert btn.style.body_color.startswith("#")


@screenshot
def test_extract_texts(
    screenshot_processor: ScreenshotProcessor, demo_screenshots: list[str]
) -> None:
    """Test extracting texts from screenshots."""
    for file_path in demo_screenshots:
        texts = screenshot_processor.extract_texts(file_path)

        # We should find some text
        assert len(texts) > 0

        # Check text properties
        for text in texts:
            assert isinstance(text, str)
            assert len(text.strip()) > 0  # Non-empty after stripping whitespace


@screenshot
def test_load_into_tree(
    screenshot_processor: ScreenshotProcessor, demo_screenshots: list[str]
) -> None:
    """Test loading screenshots into tree structure."""
    for file_path in demo_screenshots:
        tree = screenshot_processor.load_into_tree(file_path)

        # Check tree structure
        assert len(tree.pages) > 0
        page = next(iter(tree.pages.values()))

        # Check page properties
        assert isinstance(page, AACPage)
        assert isinstance(page.id, str)
        assert page.id.startswith("screenshot_")
        assert len(page.buttons) > 0


@screenshot
def test_save_from_tree(
    screenshot_processor: ScreenshotProcessor, demo_screenshots: list[str]
) -> None:
    """Test that save_from_tree raises NotImplementedError."""
    tree = screenshot_processor.load_into_tree(demo_screenshots[0])
    with pytest.raises(NotImplementedError):
        screenshot_processor.save_from_tree(tree, "output.png")



import os
import shutil
import sqlite3
import tempfile
from abc import abstractmethod
from pathlib import Path
from sqlite3 import Connection, Cursor
from threading import Lock
from typing import Any, Optional, Union

from .base_processor import AACProcessor
from .tree_structure import AACButton, AACPage, ButtonType


class SQLiteProcessor(AACProcessor):
    """Base class for processors that handle SQLite database files."""

    def __init__(self) -> None:
        """Initialize the SQLite processor."""
        super().__init__()
        self.collected_texts = []
        self._db_lock = Lock()
        self._query_cache: dict[str, list[tuple[Any, ...]]] = {}
        self._temp_dirs: list[str] = []  # Track temporary directories for cleanup
        self.file_path: Optional[str] = None  # Store the current file path
        self._conn: Optional[Connection] = None
        self._debug_output = print  # Default debug output

    def create_temp_dir(self) -> str:
        """Create a temporary directory and track it for cleanup.

        The directory will be automatically cleaned up when cleanup() is called.

        Returns:
            str: Path to created temporary directory.
        """
        temp_dir = tempfile.mkdtemp()
        self._temp_dirs.append(temp_dir)
        return temp_dir

    def cleanup(self) -> None:
        """Clean up temporary directories."""
        for temp_dir in self._temp_dirs:
            if os.path.exists(temp_dir):
                shutil.rmtree(temp_dir)
        self._temp_dirs = []

    def process_texts(
        self,
        file_path: str,
        translations: Optional[dict[str, str]] = None,
        output_path: Optional[str] = None,
    ) -> Union[list[str], str, None]:
        """Process texts in SQLite database.

        Args:
            file_path (str): Path to the SQLite database file.
            translations (Optional[Dict[str, str]]): Dictionary of translations.
            output_path (Optional[str]): Path where to save the translated file.

        Returns:
            Union[List[str], str, None]: List of texts if extracting,
            path to translated file if translating, None if error.
        """
        try:
            # Reset state for new translation
            self.collected_texts = []
            self.set_source_file(file_path)

            # Prepare workspace and get working directory
            workspace = self._prepare_workspace(file_path)

            # Process the files
            result = self.process_files(workspace, translations)

            if translations is None:
                return self.collected_texts

            if result and output_path:
                self._create_output(workspace, output_path)
                return output_path

            return None

        finally:
            self.cleanup_temp_files()

    def _connect(self, db_path: Union[str, Path]) -> Connection:
        """Connect to SQLite database"""
        if not os.path.exists(db_path):
            raise FileNotFoundError(f"Database file not found: {db_path}")
        return sqlite3.connect(db_path)

    def _execute_query(
        self, query: str, params: Optional[tuple] = None
    ) -> list[tuple[Any, ...]]:
        """Execute a query and return results"""
        if not self._conn:
            raise RuntimeError("No database connection")

        cursor: Cursor = self._conn.cursor()
        if params:
            cursor.execute(query, params)
        else:
            cursor.execute(query)
        return cursor.fetchall()

    def _execute_many(self, query: str, params: list[tuple]) -> None:
        """Execute multiple queries with different parameters"""
        if not self._conn:
            raise RuntimeError("No database connection")

        cursor: Cursor = self._conn.cursor()
        cursor.executemany(query, params)
        self._conn.commit()

    @abstractmethod
    def process_files(
        self, directory: str, translations: Optional[dict[str, str]] = None
    ) -> Optional[str]:
        """Process files in the directory - implement in child class.

        Args:
            directory (str): Directory containing the files to process.
            translations (Optional[Dict[str, str]]): Dictionary of translations.

        Returns:
            Optional[str]: Path to translated file if successful, None otherwise.
        """
        pass

    def get_output_path(self, target_lang: Optional[str] = None) -> str:
        """Get output path for translated file.

        Args:
            target_lang (Optional[str]): Target language code.

        Returns:
            str: Path where translated file should be saved.

        Raises:
            ValueError: If no file path is set.
        """
        if not self.file_path:
            raise ValueError("No file path set")
        dir_name = os.path.dirname(self.file_path)
        basename = os.path.basename(self.file_path)
        base_name = os.path.splitext(basename)[0]
        ext = os.path.splitext(self.file_path)[1]
        return os.path.join(dir_name, f"{base_name}_{target_lang}{ext}")

    def _convert_page_to_obf(self, page: AACPage) -> dict:
        """Convert page to OBF format - common implementation for SQLite processors.

        Args:
            page (AACPage): Page to convert.

        Returns:
            dict: OBF format data.
        """
        return {
            "id": page.id,
            "name": page.name,
            "grid": {"rows": page.grid_size[0], "columns": page.grid_size[1]},
            "buttons": [
                {
                    "id": button.id,
                    "label": button.label,
                    "vocalization": button.vocalization,
                    "load_board": (
                        {"id": button.target_page_id}
                        if button.type == ButtonType.NAVIGATE
                        else None
                    ),
                }
                for button in page.buttons
            ],
        }

    def _convert_obf_to_page(self, obf_data: dict) -> AACPage:
        """Convert OBF data to AACPage.

        Args:
            obf_data (dict): OBF format data.

        Returns:
            AACPage: Converted page.
        """
        page = AACPage(
            id=obf_data.get("id", ""),
            name=obf_data.get("name", ""),
            grid_size=(
                obf_data.get("grid", {}).get("rows", 1),
                obf_data.get("grid", {}).get("columns", 1),
            ),
        )

        # Process buttons
        for button_data in obf_data.get("buttons", []):
            if not button_data:  # Skip if button_data is None
                continue

            # Get button type and target
            button_type = ButtonType.SPEAK
            target_page_id = None

            # Handle navigation buttons
            load_board = button_data.get("load_board", {})
            if load_board:
                button_type = ButtonType.NAVIGATE
                target_page_id = load_board.get("id")

            # Handle action buttons
            actions = button_data.get("actions", [])
            if actions:
                button_type = ButtonType.ACTION

            # Create button
            button = AACButton(
                id=button_data.get("id", ""),
                label=button_data.get("label", ""),
                type=button_type,
                position=(0, 0),  # Default position
                vocalization=button_data.get("vocalization"),
                target_page_id=target_page_id,
            )

            # Add actions if present
            if actions:
                button.action = actions[0]  # Take first action

            page.buttons.append(button)

        return page

    def debug(self, message: str) -> None:
        """Output debug message.

        Args:
            message (str): Message to output.
        """
        if self._debug_output:
            self._debug_output(f"{self.__class__.__name__}: {message}")

    def _debug_print(self, message: str) -> None:
        """Print debug message.

        Args:
            message (str): Message to print.
        """
        if self._debug_output:
            self._debug_output(message)

    def set_source_file(self, file_path: str) -> None:
        """Set source file path.

        Args:
            file_path: Path to source file.
        """
        super().set_source_file(file_path)  # Call parent implementation
        self.file_path = file_path  # Set file_path for SQLite operations



"""Example script showing how to convert a screenshot to OBF format."""

from pathlib import Path

from aac_processors import viewer
from aac_processors.coughdrop_processor import CoughDropProcessor  # For OBF format
from aac_processors.optional.screenshot_processor import ScreenshotProcessor

# First install the package with dependencies:
# uv pip install -e ".[dev,screenshot]"

# Initialize processors
screenshot_processor = ScreenshotProcessor(
    save_debug_images=True
)  # Set to True to save debug images
coughdrop_processor = CoughDropProcessor()

# Input screenshot
input_file = Path("examples/demofiles/proloquo2go.png")
output_file = input_file.with_suffix(".obf")  # OBF extension

# Create tree from screenshot
tree = screenshot_processor.load_into_tree(str(input_file))

# Show what we detected
print("\nDetected board structure:")
viewer.print_tree(tree)

# Print detailed page info
page = list(tree.pages.values())[0]  # First page
print("\nDetailed page information:")
print(f"Grid size: {page.grid_size}")
print(f"Expected buttons: {7 * 11}, Found: {len(page.buttons)}\n")

print("Buttons with text:")
for btn in page.buttons:
    print(f"- Position {btn.position}: '{btn.label}' (color: {btn.style.body_color})")

# Then save tree as OBF format
coughdrop_processor.save_from_tree(tree, str(output_file))

print(f"\nConverted {input_file} to {output_file}")



import os
import os.path
import shutil
import sqlite3
import zipfile
from typing import Optional, Union

from .sqlite_processor import SQLiteProcessor
from .tree_structure import AACButton, AACPage, AACTree, ButtonType


class TouchChatProcessor(SQLiteProcessor):
    """Processor for TouchChat files (.ce)."""

    def __init__(self) -> None:
        """Initialize TouchChat processor."""
        super().__init__()
        self.is_archive = True  # CE files are archives containing SQLite DB
        self.file_path: Optional[str] = None
        self.original_filename: Optional[str] = None
        self.original_file_path: Optional[str] = None

    def can_process(self, file_path: str) -> bool:
        """Check if file is a TouchChat export.

        Args:
            file_path (str): Path to the file to check.

        Returns:
            bool: True if file is a TouchChat export.
        """
        return file_path.lower().endswith((".ce", ".wf"))

    def process_texts(
        self,
        file_path: str,
        translations: Optional[dict[str, str]] = None,
        output_path: Optional[str] = None,
    ) -> Union[list[str], str, None]:
        """Process texts from a TouchChat file.

        Process and optionally translate texts from a TouchChat file.

        Args:
            file_path: Path to the TouchChat file.
            translations: Optional dictionary of translations.
            output_path: Optional path where to save the translated file.

        Returns:
            List[str]: List of extracted texts if no translations provided.
            str: Path to translated file if translations provided.
            None: If an error occurs during processing.
        """
        try:
            # Reset state for new translation
            self.collected_texts = []
            self.file_path = file_path
            self.original_file_path = file_path
            self.original_filename = os.path.splitext(os.path.basename(file_path))[0]
            self.debug(f"Processing file: {file_path}")
            self.debug(f"Original filename: {self.original_filename}")

            # Use existing temp dir if available, otherwise create new one
            if not self._temp_dirs:
                self._temp_dirs.append(self.create_temp_dir())
            temp_dir = self._temp_dirs[0]
            self.debug(f"Using temp dir: {temp_dir}")

            # Clean temp dir before use
            for item in os.listdir(temp_dir):
                item_path = os.path.join(temp_dir, item)
                if os.path.isfile(item_path) and not item_path == file_path:
                    os.remove(item_path)
                elif os.path.isdir(item_path):
                    shutil.rmtree(item_path)

            # Extract the archive
            self.debug("Extracting archive...")
            self.extract_archive(file_path, temp_dir)
            self.debug(f"Temp dir contents after extraction: {os.listdir(temp_dir)}")

            # Process the files
            self.debug("Processing files...")
            result = self.process_files(temp_dir, translations)
            self.debug(f"Process files result: {result}")

            if translations is None:
                # self.debug(f"Returning collected texts: {self.collected_texts}")
                return self.collected_texts

            if result:
                # Create output path if not provided
                target_lang = translations.get("target_lang", "translated")
                final_output = output_path or os.path.join(
                    os.path.dirname(file_path),
                    f"{self.original_filename}_{target_lang}.ce",
                )
                self.debug(f"Creating output file: {final_output}")

                # Create CE file
                with zipfile.ZipFile(
                    final_output, "w", zipfile.ZIP_DEFLATED
                ) as zip_ref:
                    # First, copy all files from original archive except .c4v
                    with zipfile.ZipFile(file_path, "r") as orig_zip:
                        for item in orig_zip.namelist():
                            if not item.endswith(".c4v"):
                                self.debug(f"Copying original file: {item}")
                                zip_ref.writestr(item, orig_zip.read(item))

                    # Find and add the translated c4v file
                    c4v_found = False
                    for root, _, files in os.walk(result):
                        for file in files:
                            if file.endswith(".c4v"):
                                file_path = os.path.join(root, file)
                                self.debug(f"Adding translated c4v file: {file_path}")
                                # Get original c4v filename from the archive
                                with zipfile.ZipFile(
                                    self.original_file_path, "r"
                                ) as orig_zip:
                                    orig_c4v = next(
                                        (
                                            name
                                            for name in orig_zip.namelist()
                                            if name.endswith(".c4v")
                                        ),
                                        None,
                                    )
                                    if orig_c4v:
                                        zip_ref.write(
                                            file_path, orig_c4v
                                        )  # Use original path
                                    else:
                                        zip_ref.write(
                                            file_path, os.path.basename(file_path)
                                        )
                                c4v_found = True
                                break
                        if c4v_found:
                            break

                    if not c4v_found:
                        self.debug("No .c4v file found to add to archive")
                        return None

                self.debug(f"Successfully created output file: {final_output}")
                return final_output

            self.debug("No result from process_files")
            return None

        except Exception as e:
            self.debug(f"Error processing texts: {e}")
            return None

    def check_is_archive(self, file_path: Optional[str]) -> bool:
        """Check if file is a TouchChat archive.

        Args:
            file_path (Optional[str]): Path to file to check.

        Returns:
            bool: True if file is a valid TouchChat archive.
        """
        if not file_path:
            return False

        # Check if it's a .ce or .wf file
        if not file_path.lower().endswith((".ce", ".wf")):
            return False

        # Verify it's a valid ZIP file
        try:
            with zipfile.ZipFile(file_path, "r") as zf:
                # Try reading the file list
                zf.namelist()
                return True
        except (zipfile.BadZipFile, OSError):
            self.debug(f"File {file_path} is not a valid ZIP archive")
            return False

    def extract_archive(self, file_path: str, target_dir: str) -> None:
        """Extract TouchChat .ce or .wf archive.

        Args:
            file_path (str): Path to .ce or .wf file
            target_dir (str): Directory to extract to
        """
        self.debug(
            f"Extract archive called with file: {file_path}, target: {target_dir}"
        )
        self.debug(f"File exists check: {os.path.exists(file_path)}")
        self.debug(f"File size: {os.path.getsize(file_path)} bytes")
        self.debug(f"Target dir exists check: {os.path.exists(target_dir)}")

        try:
            with zipfile.ZipFile(file_path, "r") as zip_ref:
                # List all files in archive
                files = zip_ref.namelist()
                self.debug(f"Files in archive: {files}")

                # Look for any .c4v file
                c4v_file = None
                for f in files:
                    if f.endswith(".c4v"):
                        c4v_file = f
                        self.debug(f"Found .c4v file in archive: {c4v_file}")
                        break

                if not c4v_file:
                    self.debug("No .c4v file found in archive")
                    return

                # Extract the .c4v file
                self.debug(f"Extracting {c4v_file} to {target_dir}")
                zip_ref.extract(c4v_file, target_dir)
                self.debug(
                    "Extraction complete. "
                    f"Target dir contents: {os.listdir(target_dir)}"
                )

        except zipfile.BadZipFile as e:
            self.debug(
                f"Failed to extract .ce file - not a valid ZIP archive: {str(e)}"
            )
            # If extraction fails, copy the file as-is
            shutil.copy2(file_path, target_dir)
        except Exception as e:
            self.debug(f"Unexpected error during extraction: {str(e)}")
            raise

    def process_files(
        self, directory: str, translations: Optional[dict[str, str]] = None
    ) -> Optional[str]:
        """Process files in directory.

        Args:
            directory (str): Path to directory containing files.
            translations (Optional[Dict[str, str]]): Dictionary of translations.

        Returns:
            Optional[str]: Path to processed file if successful, None if error.
        """
        try:
            # Find any .c4v file
            c4v_file = None
            self.debug(f"Process files called with directory: {directory}")
            self.debug(f"Directory exists check: {os.path.exists(directory)}")
            self.debug(f"Directory contents: {os.listdir(directory)}")

            for file in os.listdir(directory):
                if file.endswith(".c4v"):
                    c4v_file = os.path.join(directory, file)
                    self.debug(f"Found .c4v file: {c4v_file}")
                    self.debug(f"C4v file exists check: {os.path.exists(c4v_file)}")
                    self.debug(f"C4v file size: {os.path.getsize(c4v_file)} bytes")
                    break

            if not c4v_file:
                self.debug("No .c4v file found in directory")
                return None

            # Create new database with translations
            new_db_path = os.path.join(directory, "translated.c4v")
            shutil.copy2(c4v_file, new_db_path)
            self.debug(f"Created new database at: {new_db_path}")

            # Connect to database
            conn = sqlite3.connect(new_db_path)
            cursor = conn.cursor()

            if translations:
                self.debug(f"Processing translations: {translations}")
                modified = False

                # Update button labels and messages
                for original, translated in translations.items():
                    if original == "target_lang":
                        continue
                    self.debug(f"Translating '{original}' to '{translated}'")

                    # Update button labels
                    cursor.execute(
                        """
                        UPDATE buttons
                        SET label = ?
                        WHERE label = ?
                        """,
                        (translated, original),
                    )
                    if cursor.rowcount > 0:
                        modified = True
                        self.debug(f"Updated {cursor.rowcount} button labels")

                    # Update button messages
                    cursor.execute(
                        """
                        UPDATE buttons
                        SET message = ?
                        WHERE message = ?
                        """,
                        (translated, original),
                    )
                    if cursor.rowcount > 0:
                        modified = True
                        self.debug(f"Updated {cursor.rowcount} exact button messages")

                    # Update button messages WITH TRAILING SPACES/CHARACTERS
                    cursor.execute(
                        """
                        UPDATE buttons
                        SET message = ? || SUBSTR(message, LENGTH(?) + 1)
                        WHERE message LIKE ? || ' %' OR message LIKE ? || ',%' OR message LIKE ? || '!%' OR message LIKE ? || '.%'
                        """,
                        (translated, original, original, original, original, original),
                    )
                    if cursor.rowcount > 0:
                        modified = True
                        self.debug(
                            f"Updated {cursor.rowcount} button messages with trailing characters"
                        )

                    # Update page names
                    cursor.execute(
                        """
                        UPDATE resources
                        SET name = ?
                        WHERE name = ?
                        """,
                        (translated, original),
                    )
                    if cursor.rowcount > 0:
                        modified = True
                        self.debug(f"Updated {cursor.rowcount} resource names")

                # Commit changes and close connection
                conn.commit()
                conn.close()

                if modified:
                    self.debug("Changes were made, replacing original database")
                    # Remove the original database
                    os.remove(c4v_file)
                    # Move the new database to the original location
                    shutil.move(new_db_path, c4v_file)
                    self.debug(f"Moved translated database to: {c4v_file}")
                    return directory
                else:
                    self.debug("No changes were made to the database")
                    # Clean up unused translated database
                    os.remove(new_db_path)
                    return None

            else:
                # Collect texts
                cursor.execute(
                    """
                    SELECT DISTINCT label, message
                    FROM buttons
                    WHERE label IS NOT NULL OR message IS NOT NULL
                    """
                )
                for label, message in cursor.fetchall():
                    if label and label.strip():
                        self.collected_texts.append(label.strip())
                    if message and message.strip():
                        self.collected_texts.append(message.strip())

                conn.close()
                return directory

            return None

        except Exception as e:
            self.debug(f"Error processing files: {e}")
            if "conn" in locals():
                conn.close()
            return None

    def extract_texts(self, file_path: str) -> list[str]:
        """Extract texts from TouchChat file.

        Args:
            file_path (str): Path to the file to extract texts from.

        Returns:
            List[str]: List of extracted texts.
        """
        try:
            # Reset state for new extraction
            self.collected_texts = []
            self.set_source_file(file_path)

            # Prepare workspace
            workspace = self._prepare_workspace(file_path)

            # Process the files
            self.process_files(workspace, None)
            return self.collected_texts

        except Exception as e:
            msg = f"Error extracting texts: {e}"
            self.debug(msg)
            return []

    def _check_database_schema(self, cursor: sqlite3.Cursor) -> None:
        """Check and create database schema if needed.

        Args:
            cursor (sqlite3.Cursor): Database cursor.
        """
        # Check if tables exist
        cursor.execute(
            """
            SELECT name FROM sqlite_master
            WHERE type='table' AND name IN (
                'special_pages', 'pages', 'resources', 'buttons',
                'button_boxes', 'button_box_instances', 'button_box_cells',
                'actions', 'action_data'
            )
            """
        )
        existing_tables = {row[0] for row in cursor.fetchall()}

        # Create missing tables
        if "special_pages" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE special_pages (
                    id INTEGER PRIMARY KEY,
                    name TEXT,
                    page_id INTEGER
                )
                """
            )

        if "pages" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE pages (
                    id INTEGER PRIMARY KEY,
                    resource_id INTEGER
                )
                """
            )

        if "resources" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE resources (
                    id INTEGER PRIMARY KEY,
                    rid TEXT,
                    name TEXT,
                    type INTEGER
                )
                """
            )

        if "buttons" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE buttons (
                    id INTEGER PRIMARY KEY,
                    resource_id INTEGER,
                    label TEXT,
                    message TEXT,
                    page_id INTEGER
                )
                """
            )

        if "button_boxes" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE button_boxes (
                    id INTEGER PRIMARY KEY,
                    init_size_x INTEGER,
                    init_size_y INTEGER
                )
                """
            )

        if "button_box_instances" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE button_box_instances (
                    id INTEGER PRIMARY KEY,
                    button_box_id INTEGER,
                    page_id INTEGER
                )
                """
            )

        if "button_box_cells" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE button_box_cells (
                    id INTEGER PRIMARY KEY,
                    button_box_id INTEGER,
                    resource_id INTEGER,
                    location INTEGER,
                    span_x INTEGER DEFAULT 1,
                    span_y INTEGER DEFAULT 1
                )
                """
            )

        if "actions" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE actions (
                    id INTEGER PRIMARY KEY,
                    resource_id INTEGER,
                    code INTEGER
                )
                """
            )

        if "action_data" not in existing_tables:
            cursor.execute(
                """
                CREATE TABLE action_data (
                    id INTEGER PRIMARY KEY,
                    action_id INTEGER,
                    key INTEGER,
                    value TEXT
                )
                """
            )

    def create_translated_file(
        self, file_path: str, translations: dict[str, str]
    ) -> str:
        """Create a translated version of the TouchChat file.

        Args:
            file_path: Path to the TouchChat file.
            translations: Dictionary of translations.

        Returns:
            Path to the translated file.
        """
        # Load the tree structure
        tree = self.load_into_tree(file_path)

        # Apply translations
        for page in tree.pages.values():
            for button in page.buttons:
                if button.label in translations:
                    button.label = translations[button.label]
                if button.vocalization in translations:
                    button.vocalization = translations[button.vocalization]

        # Save the translated tree
        target_lang = translations.get("target_lang", "translated")
        base_name, ext = os.path.splitext(os.path.basename(file_path))
        output_name = (
            f"{base_name}_{target_lang}{ext}"  # Use the same extension as input
        )
        output_path = os.path.join(os.path.dirname(file_path), output_name)
        self.save_from_tree(tree, output_path)
        return output_path

    def save_from_tree(self, tree: AACTree, output_path: str) -> None:
        """Save tree to TouchChat format.

        Args:
            tree (AACTree): Tree structure to save.
            output_path (str): Path where to save the file.
        """
        # Create a temporary directory for the database
        workspace = self.get_session_workspace()
        db_path = os.path.join(workspace, "output.c4v")

        # Create database
        conn = sqlite3.connect(db_path)
        cursor = conn.cursor()

        try:
            # Create tables
            self._check_database_schema(cursor)

            # Process each page
            for page_id, page in tree.pages.items():
                # Insert page resource
                cursor.execute(
                    """
                    INSERT INTO resources (rid, name, type)
                    VALUES (?, ?, ?)
                    """,
                    (f"page_{page_id}", page.name, 1),  # type 1 = page
                )
                page_resource_id = cursor.lastrowid

                # Insert page
                cursor.execute(
                    """
                    INSERT INTO pages (resource_id)
                    VALUES (?)
                    """,
                    (page_resource_id,),
                )
                db_page_id = cursor.lastrowid

                # Create button box for the page's grid
                cursor.execute(
                    """
                    INSERT INTO button_boxes (init_size_x, init_size_y)
                    VALUES (?, ?)
                    """,
                    (page.grid_size[1], page.grid_size[0]),  # x = cols, y = rows
                )
                button_box_id = cursor.lastrowid

                # Link button box to page
                cursor.execute(
                    """
                    INSERT INTO button_box_instances (button_box_id, page_id)
                    VALUES (?, ?)
                    """,
                    (button_box_id, db_page_id),
                )

                # Process each button
                for button in page.buttons:
                    # Calculate button location in grid
                    row, col = button.position
                    location = row * page.grid_size[1] + col

                    # Insert button resource
                    rid = f"btn_{button.id}"
                    cursor.execute(
                        """
                        INSERT INTO resources
                        (rid, name, type)
                        VALUES (?, ?, ?)
                        """,
                        (rid, button.label, 2),  # type 2 = button
                    )
                    button_resource_id = cursor.lastrowid

                    # Insert button
                    cursor.execute(
                        """
                        INSERT INTO buttons
                        (resource_id, label, message, page_id)
                        VALUES (?, ?, ?, ?)
                        """,
                        (
                            button_resource_id,
                            button.label,
                            button.vocalization,
                            db_page_id,
                        ),
                    )

                    # Insert button cell
                    cursor.execute(
                        """
                        INSERT INTO button_box_cells
                        (button_box_id, resource_id, location)
                        VALUES (?, ?, ?)
                        """,
                        (button_box_id, button_resource_id, location),
                    )

                    # If it's a navigation button, add action
                    if button.type == ButtonType.NAVIGATE and button.target_page_id:
                        cursor.execute(
                            """
                            INSERT INTO actions
                            (resource_id, code)
                            VALUES (?, ?)
                            """,
                            (button_resource_id, 1),  # code 1 = navigate
                        )
                        action_id = cursor.lastrowid

                        cursor.execute(
                            """
                            INSERT INTO action_data
                            (action_id, key, value)
                            VALUES (?, ?, ?)
                            """,
                            (
                                action_id,
                                1,
                                button.target_page_id,
                            ),  # key 1 = target page
                        )

                # Set home page if this is the root page
                if page_id == tree.root_id:
                    cursor.execute(
                        """
                        INSERT INTO special_pages (name, page_id)
                        VALUES (?, ?)
                        """,
                        ("Home", db_page_id),
                    )

            conn.commit()

            # Create CE file
            with zipfile.ZipFile(output_path, "w", zipfile.ZIP_DEFLATED) as zip_ref:
                zip_ref.write(db_path, "output.c4v")

        except Exception as e:
            msg = f"Error saving tree: {e}"
            self.debug(msg)
            raise
        finally:
            conn.close()

    def load_into_tree(self, file_path: str) -> AACTree:
        """Load TouchChat file into tree structure.

        Args:
            file_path (str): Path to the file to load.

        Returns:
            AACTree: Tree structure representing the file contents.

        Raises:
            ValueError: If the file is not a valid TouchChat archive.
            Exception: If there is an error loading the file.
        """
        try:
            tree = AACTree()
            workspace = self._prepare_workspace(file_path)

            # Find the .c4v file
            c4v_file = None
            for file in os.listdir(workspace):
                if file.endswith(".c4v"):
                    c4v_file = os.path.join(workspace, file)
                    break

            if not c4v_file:
                raise ValueError("No .c4v file found in archive")

            # Connect to database
            conn = sqlite3.connect(c4v_file)
            cursor = conn.cursor()

            # Ensure database schema exists
            self._check_database_schema(cursor)

            # First find home page from special_pages
            cursor.execute(
                """
                SELECT page_id
                FROM special_pages
                WHERE name = 'Home'
                """
            )
            result = cursor.fetchone()
            if result:
                tree.root_id = str(result[0])

            # Get all pages
            cursor.execute(
                """
                SELECT p.id, r.name, bb.init_size_x, bb.init_size_y
                FROM pages p
                JOIN resources r ON p.resource_id = r.id
                JOIN button_box_instances bbi ON bbi.page_id = p.id
                JOIN button_boxes bb ON bbi.button_box_id = bb.id
                """
            )

            for page_id, page_name, grid_x, grid_y in cursor.fetchall():
                page = AACPage(
                    id=str(page_id),
                    name=page_name,
                    grid_size=(
                        grid_y or 1,
                        grid_x or 1,
                    ),  # TouchChat uses x,y but we use rows,cols
                )

                # Get buttons for this page
                cursor.execute(
                    """
                    SELECT b.id, b.label, b.message,
                        bbc.location, bbc.span_x, bbc.span_y,
                        a.code
                    FROM button_box_instances bbi
                    JOIN button_boxes bb ON bbi.button_box_id = bb.id
                    JOIN button_box_cells bbc ON bbc.button_box_id = bb.id
                    JOIN buttons b ON b.resource_id = bbc.resource_id
                    LEFT JOIN actions a ON a.resource_id = b.resource_id
                    WHERE bbi.page_id = ?
                    """,
                    (page_id,),
                )

                for row in cursor.fetchall():
                    (
                        button_id,
                        label,
                        message,
                        location,
                        span_x,
                        span_y,
                        action_code,
                    ) = row

                    # Calculate position from location
                    y = location // grid_x if grid_x else 0
                    x = location % grid_x if grid_x else 0

                    # Determine button type based on action code
                    button_type = ButtonType.SPEAK
                    target_page_id = None

                    if action_code == 1:  # Navigate action
                        button_type = ButtonType.NAVIGATE
                        # Get target page from action data
                        cursor.execute(
                            """
                            SELECT value
                            FROM action_data
                            WHERE action_id = ? AND key = 1
                            """,
                            (button_id,),
                        )
                        target_result = cursor.fetchone()
                        if target_result:
                            target_page_id = str(target_result[0])

                    button = AACButton(
                        id=str(button_id),
                        label=label or "",
                        type=button_type,
                        position=(y, x),
                        target_page_id=target_page_id,
                        vocalization=message,
                    )
                    page.buttons.append(button)

                tree.add_page(page)

            conn.close()
            return tree

        except Exception as e:
            msg = f"Error loading TouchChat file: {e}"
            self.debug(msg)
            raise

    def process_translations(
        self, input_path: str, translations: dict[str, str], output_path: str
    ) -> None:
        """Process translations and create translated file.

        Copy the input file and update its contents with translations.

        Args:
            input_path: Path to input file.
            translations: Dictionary of translations.
            output_path: Path to output file.
        """
        # Copy original file to output path
        shutil.copy2(input_path, output_path)

        # Update translations in the copied file
        self.create_translated_file(output_path, translations)



import os
import shutil

import pytest

from aac_processors.dot_processor import DotProcessor
from aac_processors.tree_structure import AACButton, AACPage, ButtonType


def test_can_process() -> None:
    processor = DotProcessor()
    assert processor.can_process("test.dot")
    assert processor.can_process("test.gv")
    assert not processor.can_process("test.txt")


def test_load_tree(test_dot_file: str) -> None:
    processor = DotProcessor()
    tree = processor.load_into_tree(test_dot_file)

    # Verify pages
    assert len(tree.pages) == 4  # Four nodes

    # Find home page by node label
    home_page = next((p for p in tree.pages.values() if p.name == "Home Page"), None)
    assert home_page is not None
    assert len(home_page.buttons) == 3  # Three outgoing edges

    # Verify buttons
    button_labels = [b.label for b in home_page.buttons]
    assert "Go to About" in button_labels
    assert "Go to Contact" in button_labels
    assert "View Products" in button_labels

    # Check button types
    for button in home_page.buttons:
        assert button.type == ButtonType.NAVIGATE


def test_save_tree(test_dot_file: str, temp_dir: str) -> None:
    import os

    processor = DotProcessor()
    tree = processor.load_into_tree(test_dot_file)

    # Save tree to new file
    output_path = os.path.join(temp_dir, "output.dot")
    processor.save_from_tree(tree, output_path)

    # Verify the saved file
    assert os.path.exists(output_path)

    # Check content
    with open(output_path) as f:
        content = f.read()

    # The node IDs might be different now, but the labels should be present
    assert 'label="Home Page"' in content
    assert 'label="About"' in content
    assert 'label="Contact"' in content
    assert 'label="Products"' in content

    # Verify edges - check for the labels which should be in the file
    assert 'label="Go to About"' in content
    assert 'label="Go to Contact"' in content
    assert 'label="View Products"' in content
    assert 'label="Back to Home"' in content


@pytest.mark.slow
def test_translation(test_dot_file: str, temp_dir: str) -> None:
    """Test translation of DOT file"""
    processor = DotProcessor()

    # Extract texts
    texts = processor.extract_texts(test_dot_file)

    # Check extracted texts
    assert "Home Page" in texts
    assert "About" in texts
    assert "Contact" in texts
    assert "Products" in texts
    assert "Go to About" in texts
    assert "Go to Contact" in texts
    assert "View Products" in texts
    assert "Back to Home" in texts

    # Create translations
    translations = {
        "Home Page": "Página Principal",
        "About": "Acerca de",
        "Contact": "Contacto",
        "Products": "Productos",
        "Go to About": "Ir a Acerca de",
        "Go to Contact": "Ir a Contacto",
        "View Products": "Ver Productos",
        "Back to Home": "Volver a Inicio",
    }

    # Process translations
    output_path = os.path.join(temp_dir, "translated.dot")
    result = processor.process_texts(test_dot_file, translations, output_path)

    # Check result
    assert result
    assert os.path.exists(output_path)

    # Verify translated content
    with open(output_path) as f:
        content = f.read()

    assert "Página Principal" in content
    assert "Acerca de" in content
    assert "Contacto" in content
    assert "Productos" in content
    assert "Ir a Acerca de" in content
    assert "Ir a Contacto" in content
    assert "Ver Productos" in content
    assert "Volver a Inicio" in content


@pytest.mark.integration
def test_dot_workflow(test_dot_file: str, temp_dir: str) -> None:
    """Test full workflow with DOT files"""
    import os

    # Setup
    processor = DotProcessor()
    test_file = os.path.join(temp_dir, "test_workflow.dot")
    output_file = os.path.join(temp_dir, "output_workflow.dot")

    # Copy test file
    shutil.copy2(test_dot_file, test_file)

    # Load into tree
    tree = processor.load_into_tree(test_file)
    assert tree is not None

    # Modify tree
    new_page_id = "new_page"
    new_page = AACPage(id=new_page_id, name="New Page", grid_size=(3, 3))
    tree.add_page(new_page)

    # Get home page by node label
    home_page = next((p for p in tree.pages.values() if p.name == "Home Page"), None)
    assert home_page is not None

    # Add button to home page
    new_button = AACButton(
        id="new_button",
        label="Go to New Page",
        type=ButtonType.NAVIGATE,
        target_page_id=new_page_id,
    )
    home_page.buttons.append(new_button)

    # Save modified tree
    processor.save_from_tree(tree, output_file)

    # Verify output
    assert os.path.exists(output_file)

    # Load modified tree
    modified_tree = processor.load_into_tree(output_file)
    assert modified_tree is not None

    # Verify new page exists
    assert "New Page" in [p.name for p in modified_tree.pages.values()]

    # Verify new button exists
    new_home_page = next(
        (p for p in modified_tree.pages.values() if p.name == "Home Page"), None
    )
    assert new_home_page is not None
    assert "Go to New Page" in [b.label for b in new_home_page.buttons]



import json
import os
import tempfile
import zipfile

import pytest

from aac_processors.coughdrop_processor import CoughDropProcessor
from aac_processors.gridset_processor import GridsetProcessor
from aac_processors.snap_processor import SnapProcessor
from aac_processors.touchchat_processor import TouchChatProcessor
from aac_processors.tree_structure import ButtonType

# Get the absolute path to the demofiles directory
DEMOFILES_DIR = os.path.join(
    os.path.dirname(os.path.dirname(__file__)), "examples", "demofiles"
)


@pytest.fixture
def demo_obf_file() -> str:
    """Path to demo OBF file"""
    return os.path.join(DEMOFILES_DIR, "project-core_es.obf")


@pytest.fixture
def demo_obz_file() -> str:
    """Path to demo OBZ file"""
    return os.path.join(DEMOFILES_DIR, "communikate-20.obz")


@pytest.fixture
def demo_gridset_file() -> str:
    """Path to demo Gridset file"""
    return os.path.join(DEMOFILES_DIR, "SimpleTest.gridset")


@pytest.fixture
def demo_snap_file() -> str:
    """Path to demo Snap file"""
    return os.path.join(DEMOFILES_DIR, "Medical Advocacy.spb")


@pytest.fixture
def demo_touchchat_file() -> str:
    """Path to demo TouchChat file"""
    return os.path.join(DEMOFILES_DIR, "WordPower42 Basic SS_UK.ce")


def test_coughdrop_extract_real_obz(demo_obz_file: str) -> None:
    """Test extracting texts from a real CoughDrop OBZ file"""
    processor = CoughDropProcessor()
    texts = processor.extract_texts(demo_obz_file)

    # Verify we got a substantial number of texts (CK20 is a large board)
    assert len(texts) > 100

    # Verify some expected CK20-specific texts are present
    expected_texts = [
        "yes",
        "no",
        "more",
        "stop",  # Common core words
        "i",
        "you",
        "it",
        "want",  # Pronouns and common verbs
        "like",
        "help",
        "go",
        "look",  # Action words
    ]
    for text in expected_texts:
        assert any(
            text.lower() == t.lower() for t in texts
        ), f"Expected text '{text}' not found"

    # Load the tree to verify board structure
    tree = processor.load_into_tree(demo_obz_file)
    assert len(tree.pages) > 1  # Should have multiple pages

    # Check some specific button types
    button_types = set()
    for page in tree.pages.values():
        for button in page.buttons:
            button_types.add(button.type)

    # CK20 should have various button types
    assert ButtonType.SPEAK in button_types
    assert ButtonType.NAVIGATE in button_types


def test_coughdrop_extract_real_obf(demo_obf_file: str) -> None:
    """Test extracting texts from a real CoughDrop OBF file"""
    processor = CoughDropProcessor()
    texts = processor.extract_texts(demo_obf_file)

    # Verify we got some texts
    assert len(texts) > 20  # Should have a decent number of texts

    # Verify some expected Spanish verbs are present
    expected_verbs = ["como", "hacer", "bien", "desear", "conseguir"]
    for verb in expected_verbs:
        assert any(
            verb.lower() == t.lower() for t in texts
        ), f"Expected verb '{verb}' not found"

    # Load the tree to verify structure
    tree = processor.load_into_tree(demo_obf_file)
    assert len(tree.pages) > 0

    # Check button properties
    for page in tree.pages.values():
        for button in page.buttons:
            if button.label:
                assert button.label.strip()  # Labels should not be empty
            if button.type == ButtonType.SPEAK:
                assert not button.target_page_id  # Speak buttons shouldn't have targets


def test_gridset_extract_real(demo_gridset_file: str) -> None:
    """Test extracting texts from a real Grid3 Gridset file"""
    processor = GridsetProcessor()
    texts = processor.extract_texts(demo_gridset_file)

    # Verify we got a reasonable number of texts
    assert len(texts) > 10

    # Verify text content
    non_empty_texts = [t for t in texts if t.strip()]
    assert len(non_empty_texts) > 0

    # Load into tree to check structure
    tree = processor.load_into_tree(demo_gridset_file)
    assert len(tree.pages) > 0

    # Check grid sizes
    for page in tree.pages.values():
        rows, cols = page.grid_size
        assert rows > 0 and cols > 0

        # Grid should be at least 4x4 for SimpleTest.gridset
        assert rows >= 4, f"Grid rows {rows} too small, expected at least 4"
        assert cols >= 4, f"Grid cols {cols} too small, expected at least 4"

        # All buttons should have valid positions
        for button in page.buttons:
            assert button.position[0] >= 0, "Button row position cannot be negative"
            assert button.position[1] >= 0, "Button column position cannot be negative"
            # Note: Button positions may exceed grid size in some cases
            # This is a known issue with Grid3 files


def test_snap_extract_real(demo_snap_file: str) -> None:
    """Test extracting texts from a real Snap file"""
    processor = SnapProcessor()
    texts = processor.extract_texts(demo_snap_file)

    # Verify text sources
    # We know from debug output that we have:
    # - 255 button labels
    # - 5 button messages
    # - 4 page titles
    assert len(texts) >= 260, f"Expected at least 260 total texts, found {len(texts)}"

    # Check for button labels and messages
    non_empty_texts = [text for text in texts if len(text.strip()) > 0]
    assert len(non_empty_texts) > 50, "Expected more than 50 non-empty texts"

    # Note: Tree structure check is skipped because the ButtonAction table
    # doesn't exist in this version of the file format


def test_touchchat_extract_real(demo_touchchat_file: str) -> None:
    """Test extracting texts from a real TouchChat file"""
    processor = TouchChatProcessor()
    texts = processor.extract_texts(demo_touchchat_file)

    # Verify we got a substantial number of texts
    assert len(texts) > 100  # WordPower is a large vocabulary

    # Check for WordPower-specific content
    wordpower_terms = ["word", "power", "basic", "i", "you", "it", "want", "like"]
    found_terms = 0
    for term in wordpower_terms:
        if any(term.lower() in t.lower() for t in texts):
            found_terms += 1
    assert found_terms >= 5, "Expected at least 5 WordPower terms"

    # Load tree to verify structure
    tree = processor.load_into_tree(demo_touchchat_file)
    assert len(tree.pages) > 0

    # Verify button types and properties
    type_counts = {t: 0 for t in ButtonType}
    for page in tree.pages.values():
        for button in page.buttons:
            type_counts[button.type] += 1

    # Should have speak buttons at minimum
    assert type_counts[ButtonType.SPEAK] > 0, "No speak buttons found"
    # Log actual button type counts for analysis
    print("Button type counts:", type_counts)


def test_coughdrop_translate_real_obz(demo_obz_file: str) -> None:
    """Test translating a real CoughDrop OBZ file"""
    processor = CoughDropProcessor()

    # First extract some texts
    texts = processor.extract_texts(demo_obz_file)

    # Create translations for common words
    common_words = ["yes", "no", "more", "stop"]
    translations = {}
    for text in texts:
        if text.lower() in common_words:
            translations[text] = f"TEST_{text}"
    translations["target_lang"] = "test"

    # Create translated file
    with tempfile.NamedTemporaryFile(suffix=".obz") as output:
        result = processor.process_texts(demo_obz_file, translations, output.name)
        assert isinstance(result, str)  # Ensure result is a string path
        assert result == output.name

        # Verify translations were applied
        translated_texts = processor.extract_texts(result)

        # Check that our specific translations were applied
        for original in translations:
            if original != "target_lang":
                expected = f"TEST_{original}"
                assert any(
                    expected == t for t in translated_texts
                ), f"Translation for '{original}' not found"

        # Verify the OBZ structure is maintained
        with zipfile.ZipFile(result, "r") as zf:
            assert "manifest.json" in zf.namelist()
            manifest = json.loads(zf.read("manifest.json"))
            assert "format" in manifest
            assert "paths" in manifest
            assert "boards" in manifest["paths"]


def test_gridset_translate_real(demo_gridset_file: str, temp_dir: str) -> None:
    """Test translating a real Grid3 Gridset file"""
    processor = GridsetProcessor()

    # First extract some texts
    texts = processor.extract_texts(demo_gridset_file)

    # Create translations for specific texts
    translations = {text: f"TEST_{text}" for text in list(texts)[:5] if text.strip()}
    translations["target_lang"] = "test"

    # Create translated file
    with tempfile.NamedTemporaryFile(suffix=".gridset") as output:
        result = processor.process_texts(demo_gridset_file, translations, output.name)
        assert isinstance(result, str)  # Ensure result is a string path
        assert result == output.name

        # Verify translations were applied
        translated_texts = processor.extract_texts(result)

        # Check specific translations
        for original in translations:
            if original != "target_lang" and original.strip():
                expected = f"TEST_{original}"
                assert any(
                    expected == t for t in translated_texts
                ), f"Translation for '{original}' not found"

        # Verify the gridset structure is maintained
        with zipfile.ZipFile(result, "r") as zf:
            files = zf.namelist()
            assert any(f.endswith("grid.xml") for f in files)


def test_snap_translate_real(demo_snap_file: str) -> None:
    """Test translating a real Snap file"""
    processor = SnapProcessor()

    # First extract some texts
    texts = processor.extract_texts(demo_snap_file)

    # Create translations for medical terms
    medical_terms = ["medical", "doctor", "nurse", "pain", "help"]
    translations = {}
    for text in texts:
        if any(term in text.lower() for term in medical_terms):
            translations[text] = f"TEST_{text}"
    translations["target_lang"] = "test"

    # Create translated file
    with tempfile.NamedTemporaryFile(suffix=".spb") as output:
        result = processor.process_texts(demo_snap_file, translations, output.name)
        assert isinstance(result, str)  # Ensure result is a string path
        assert result == output.name

        # Verify translations were applied
        translated_texts = processor.extract_texts(result)

        # Check specific translations
        for original in translations:
            if original != "target_lang":
                expected = f"TEST_{original}"
                assert any(
                    expected == t for t in translated_texts
                ), f"Translation for '{original}' not found"


def test_touchchat_translate_real(demo_touchchat_file: str) -> None:
    """Test translating a real TouchChat file"""
    processor = TouchChatProcessor()

    # First extract some texts
    texts = processor.extract_texts(demo_touchchat_file)
    print(f"Extracted texts: {texts}")  # See what texts were found

    # Create translations for common words
    common_words = ["i", "you", "it", "want", "like"]
    translations = {}
    for text in texts:
        if text.lower() in common_words:
            translations[text] = f"TEST_{text}"
    print(f"Created translations: {translations}")  # See what translations were created

    translations["target_lang"] = "test"

    # Create translated file
    with tempfile.NamedTemporaryFile(suffix=".ce") as output:
        result = processor.process_texts(demo_touchchat_file, translations, output.name)
        print(f"Process texts result: {result}")  # See the result path

        # Verify translations were applied
        translated_texts = processor.extract_texts(result)
        print(
            f"Translated texts: {translated_texts}"
        )  # See what texts are in the translated file

        # Check specific translations
        for original in translations:
            if original != "target_lang":
                expected = f"TEST_{original}"
                found = any(expected == t for t in translated_texts)
                print(
                    f"Looking for translation of '{original}' -> '{expected}': "
                    f"{'Found' if found else 'Not found'}"
                )
